from datetime import datetime

import dynatrace.environment_v2.problems as pb
from dynatrace import Dynatrace
from dynatrace.pagination import PaginatedList
from dynatrace.utils import int64_to_datetime
from dynatrace.configuration_v1.alerting_profiles import AlertingProfileStub
from dynatrace.environment_v2.monitored_entities import EntityStub, EntityId
from dynatrace.environment_v2.schemas import ManagementZone
from dynatrace.environment_v2.custom_tags import METag

PROBLEM_ID = "-1719139739592062093_1623004451641V2"
COMMENT_ID = "-7228967546616810529_1623004451641"


def test_list(dt: Dynatrace):
    problems = dt.problems.list(time_from="now-3d")

    assert isinstance(problems, PaginatedList)
    assert len(list(problems)) == 2
    assert all(isinstance(p, pb.Problem) for p in problems)


def test_get(dt: Dynatrace):
    problem = dt.problems.get(problem_id=PROBLEM_ID)

    # type checks
    assert isinstance(problem, pb.Problem)
    assert isinstance(problem.affected_entities, list)
    assert isinstance(problem.impacted_entities, list)
    assert isinstance(problem.management_zones, list)
    assert isinstance(problem.entity_tags, list)
    assert isinstance(problem.problem_filters, list)
    assert isinstance(problem.start_time, datetime)
    assert isinstance(problem.end_time, datetime)
    assert all(isinstance(ae, EntityStub) for ae in problem.affected_entities)
    assert all(isinstance(ae.entity_id, EntityId) for ae in problem.affected_entities)
    assert all(isinstance(ie, EntityStub) for ie in problem.impacted_entities)
    assert all(isinstance(ie.entity_id, EntityId) for ie in problem.impacted_entities)
    assert isinstance(problem.root_cause_entity, EntityStub)
    assert all(isinstance(mz, ManagementZone) for mz in problem.management_zones)
    assert all(isinstance(et, METag) for et in problem.entity_tags)
    assert all(isinstance(pf, AlertingProfileStub) for pf in problem.problem_filters)
    assert isinstance(problem.evidence_details, pb.EvidenceDetails)
    assert all(isinstance(e, pb.Evidence) for e in problem.evidence_details.details)
    assert all(isinstance(e.entity, EntityStub) for e in problem.evidence_details.details)
    assert all(isinstance(e.grouping_entity, EntityStub) for e in problem.evidence_details.details)
    assert isinstance(problem.recent_comments, pb.CommentList)
    assert all(isinstance(c, pb.Comment) for c in problem.recent_comments.comments)
    assert isinstance(problem.impact_analysis, pb.ImpactAnalysis)
    assert all(isinstance(i, pb.Impact) for i in problem.impact_analysis.impacts)

    # value checks
    assert problem.problem_id == PROBLEM_ID
    assert problem.display_id == "P-210617"
    assert problem.title == "Multiple infrastructure problems"
    assert problem.impact_level == pb.ImpactLevel.INFRASTRUCTURE
    assert problem.severity_level == pb.SeverityLevel.RESOURCE_CONTENTION
    assert problem.status == pb.Status.CLOSED
    assert all(ae.entity_id.id == "PROCESS_GROUP_INSTANCE-8092E71D6FBB914E" for ae in problem.affected_entities)
    assert all(ae.entity_id.type == "PROCESS_GROUP_INSTANCE" for ae in problem.affected_entities)
    assert all(ae.name == "easytravel.customer.frontend" for ae in problem.affected_entities)
    assert all(ie.entity_id.id == "PROCESS_GROUP_INSTANCE-8092E71D6FBB914E" for ie in problem.impacted_entities)
    assert all(ie.entity_id.type == "PROCESS_GROUP_INSTANCE" for ie in problem.impacted_entities)
    assert all(ie.name == "easytravel.customer.frontend" for ie in problem.impacted_entities)
    assert problem.root_cause_entity.entity_id.id == "PROCESS_GROUP-C44FB250621B8036"
    assert problem.root_cause_entity.entity_id.type == "PROCESS_GROUP"
    assert problem.root_cause_entity.name == "easytravel.customer.frontend"
    assert problem.management_zones[0].id == "8692695975020499402"
    assert problem.management_zones[0].name == "Operations Team"
    assert all(str(et.context) == "CONTEXTLESS" for et in problem.entity_tags)
    assert problem.start_time == int64_to_datetime(1622807640000)
    assert problem.end_time == int64_to_datetime(1622807820000)
    assert problem.evidence_details.total_count == 4
    assert len(problem.evidence_details.details) == 4
    assert problem.recent_comments.total_count == 2
    assert len(problem.recent_comments.comments) == 2


def test_close(dt: Dynatrace):
    close_result = dt.problems.close(problem_id=PROBLEM_ID, message="Closing this. 1234")

    # type checks
    assert isinstance(close_result, pb.ProblemCloseResult)
    assert isinstance(close_result.comment, pb.Comment)
    assert isinstance(close_result.close_timestamp, datetime)
    assert isinstance(close_result.closing, bool)

    # value checks
    assert close_result.problem_id == PROBLEM_ID
    assert close_result.comment.id == COMMENT_ID
    assert close_result.comment.created_at == int64_to_datetime(1623047022173)
    assert close_result.comment.content == "Closing this. 1234"
    assert close_result.comment.author == "radu.stefan@dynatrace.com"
    assert close_result.comment.context == "dynatrace-problem-close"
    assert close_result.close_timestamp == int64_to_datetime(1623047022173)
    assert close_result.closing == True


def test_list_comments(dt: Dynatrace):
    comments = dt.problems.list_comments(problem_id=PROBLEM_ID, page_size=20)

    assert isinstance(comments, PaginatedList)
    assert len(list(comments)) == 2
    assert all(isinstance(c, pb.Comment) for c in comments)


def test_get_comment(dt: Dynatrace):
    comment = dt.problems.get_comment(problem_id=PROBLEM_ID, comment_id=COMMENT_ID)

    assert isinstance(comment, pb.Comment)

    assert comment.id == COMMENT_ID
    assert comment.created_at == int64_to_datetime(1623047022173)
    assert comment.content == "Closing this. 1234"
    assert comment.context == "dynatrace-problem-close"
    assert comment.author == "radu.stefan@dynatrace.com"



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from enum import Enum
from typing import List, Optional, Dict, Any
from requests import Response

from dynatrace.pagination import PaginatedList
from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient
from dynatrace.configuration_v1.schemas import ConfigurationMetadata, StringComparisonOperator
from dynatrace.environment_v2.monitored_entities import EntityShortRepresentation
from dynatrace.environment_v2.custom_tags import METag


class AlertingProfileTagFilter(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.include_mode: TagFilterIncludeMode = TagFilterIncludeMode(raw_element["includeMode"])
        self.tag_filters: Optional[List[METag]] = [METag(raw_element=tag) for tag in raw_element.get("tagFilters", [])]

    def to_json(self) -> Dict[str, Any]:
        details: Dict[str, Any] = {"includeMode": str(self.include_mode)}
        if self.tag_filters:
            details["tagFilters"] = [tf.to_json() for tf in self.tag_filters]
        return details


class AlertingProfileSeverityRule(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.severity_level: SeverityLevel = SeverityLevel(raw_element["severityLevel"])
        self.tag_filter: AlertingProfileTagFilter = AlertingProfileTagFilter(raw_element=raw_element["tagFilter"])
        self.delay_in_minutes: int = raw_element["delayInMinutes"]

    def to_json(self) -> Dict[str, Any]:
        return {"severityLevel": str(self.severity_level), "tagFilter": self.tag_filter.to_json(), "delayInMinutes": self.delay_in_minutes}


class AlertingPredefinedEventFilter(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.event_type: AlertingPredefinedEvent = AlertingPredefinedEvent(raw_element["eventType"])
        self.negate: bool = raw_element["negate"]

    def to_json(self) -> Dict[str, Any]:
        return {"eventType": str(self.event_type), "negate": self.negate}


class AlertingCustomTextFilter(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.enabled: bool = raw_element["enabled"]
        self.value: str = raw_element["value"]
        self.operator: StringComparisonOperator = StringComparisonOperator(raw_element["operator"])
        self.negate: bool = raw_element["negate"]
        self.case_insensitive: bool = raw_element["caseInsensitive"]

    def to_json(self) -> Dict[str, Any]:
        return {"enabled": self.enabled, "value": self.value, "operator": str(self.operator), "negate": self.negate, "caseInsensitive": self.case_insensitive}


class AlertingCustomEventFilter(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.custom_title_filter: Optional[AlertingCustomTextFilter] = (
            AlertingCustomTextFilter(raw_element=raw_element.get("customTitleFilter")) if "customTitleFilter" in raw_element.keys() else None
        )
        self.custom_description_filter: Optional[AlertingCustomTextFilter] = (
            AlertingCustomTextFilter(raw_element=raw_element.get("customDescriptionFilter")) if "customDescriptionFilter" in raw_element.keys() else None
        )

    def to_json(self) -> Dict[str, Any]:
        details = {}
        if self.custom_title_filter:
            details["customTitleFilter"] = self.custom_title_filter.to_json()
        if self.custom_description_filter:
            details["customDescriptionFilter"] = self.custom_description_filter.to_json()
        return details


class AlertingEventTypeFilter(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.predefined_event_filter: Optional[AlertingPredefinedEventFilter] = (
            AlertingPredefinedEventFilter(raw_element=raw_element.get("predefinedEventFilter")) if "predefinedEventFilter" in raw_element.keys() else None
        )
        self.custom_event_filter: Optional[AlertingCustomEventFilter] = (
            AlertingCustomEventFilter(raw_element=raw_element.get("customEventFilter")) if "customEventFilter" in raw_element.keys() else None
        )

    def to_json(self) -> Dict[str, Any]:
        details = {}
        if self.predefined_event_filter:
            details["predefinedEventFilter"] = self.predefined_event_filter.to_json()
        if self.custom_event_filter:
            details["customEventFilter"] = self.custom_event_filter.to_json()
        return details


class AlertingProfile(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.metadata: Optional[ConfigurationMetadata] = ConfigurationMetadata(raw_element=raw_element.get("metadata"))
        self.id: Optional[str] = raw_element.get("id")
        self.display_name: str = raw_element["displayName"]
        self.rules: Optional[List[AlertingProfileSeverityRule]] = [AlertingProfileSeverityRule(raw_element=rule) for rule in raw_element.get("rules", [])]
        self.management_zone_id: Optional[str] = raw_element.get("mzId")
        self.event_type_filters: Optional[List[AlertingEventTypeFilter]] = [
            AlertingEventTypeFilter(raw_element=event_filter) for event_filter in raw_element.get("eventTypeFilters", [])
        ]

    def post(self) -> EntityShortRepresentation:
        """Creates the Alerting Profile configuration in Dynatrace (POST).

        :param alerting_profile: the Alerting Profile configuration details

        :returns EntityShortRepresentation: basic details of the created Alerting Profile

        :throws ValueError: if operation cannot be executed due to missing HTTP Client
        """
        if not self._http_client:
            raise ValueError("Object does not have an HTTP Client. Use alerting_profiles.post() instead.")
        response = self._http_client.make_request(path=AlertingProfileService.ENDPOINT, params=self.to_json(), method="POST")
        self.id = response.json().get("id")

        return EntityShortRepresentation(raw_element=response.json())

    def put(self) -> Response:
        """Updates the Alerting Profile configuration in Dynatrace (PUT).
        If the ID does not exist in Dynatrace, a new Alerting Profile will be created with the given ID.

        :param alerting_profile: the Alerting Profile configuration details

        :returns Response: HTTP Response to the request. Will contain basic details in JSON body if config is created.

        :throws ValueError: if operation cannot be executed due to missing HTTP Client
        """
        if not self._http_client:
            raise ValueError("Object does not have an HTTP Client. Use alerting_profiles.put() instead.")
        response = self._http_client.make_request(path=f"{AlertingProfileService.ENDPOINT}/{self.id}", params=self.to_json(), method="PUT")
        if response.status_code == 201:
            self.id = response.json().get("id")

        return response

    def to_json(self) -> Dict[str, Any]:
        """Get a JSON (dict) representation of this config."""
        details: Dict[str, Any] = {"displayName": self.display_name}
        if self.rules:
            details["rules"] = [r.to_json() for r in self.rules]
        if self.management_zone_id:
            details["mzId"] = self.management_zone_id
        if self.event_type_filters:
            details["eventTypeFilters"] = [etf.to_json() for etf in self.event_type_filters]
        return details


class AlertingProfileStub(EntityShortRepresentation):
    def get_full_configuration(self):
        """
        Gathers the full details of the alerting profile
        """
        if not self._http_client:
            raise ValueError("Object does not have an HTTP Client implemented.")
        response = self._http_client.make_request(f"{AlertingProfileService.ENDPOINT}/{self.id}").json()
        return AlertingProfile(self._http_client, None, response)


class AlertingProfileService:
    ENDPOINT = "/api/config/v1/alertingProfiles"

    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(self) -> PaginatedList[AlertingProfileStub]:
        """
        Lists all alerting profiles in the environmemt. No configurable parameters.
        """
        return PaginatedList(AlertingProfileStub, self.__http_client, f"{self.ENDPOINT}", list_item="values")

    def get(self, profile_id: str) -> AlertingProfile:
        """Gets the full details of the Alerting Profile referenced by ID.

        :param profile_id: ID of the alerting profile

        :returns AlertingProfile: alerting profile details
        """
        response = self.__http_client.make_request(f"{self.ENDPOINT}/{profile_id}")
        return AlertingProfile(http_client=self.__http_client, raw_element=response.json())

    def delete(self, profile_id: str) -> Response:
        """
        Delete the alerting profile with the specified id.
        """
        return self.__http_client.make_request(f"{self.ENDPOINT}/{profile_id}", method="DELETE")

    def post(self, alerting_profile: AlertingProfile) -> EntityShortRepresentation:
        """Creates the Alerting Profile configuration in Dynatrace (POST).

        :param alerting_profile: the Alerting Profile configuration details

        :returns EntityShortRepresentation: basic details of the created Alerting Profile
        """
        if not alerting_profile._http_client:
            alerting_profile._http_client = self.__http_client
        return alerting_profile.post()

    def put(self, alerting_profile: AlertingProfile) -> Response:
        """Updates the Alerting Profile configuration in Dynatrace (PUT).
        If the ID does not exist in Dynatrace, a new Alerting Profile will be created with the given ID.

        :param alerting_profile: the Alerting Profile configuration details

        :returns Response: HTTP Response to the request. Will contain basic details in JSON body if config is created.
        """
        if not alerting_profile._http_client:
            alerting_profile._http_client = self.__http_client
        return alerting_profile.put()


class TagFilterIncludeMode(Enum):
    INCLUDE_ALL = "INCLUDE_ALL"
    INCLUDE_ANY = "INCLUDE_ANY"
    NONE = "NONE"

    def __str__(self) -> str:
        return self.value


class AlertingPredefinedEvent(Enum):
    APPLICATION_ERROR_RATE_INCREASED = "APPLICATION_ERROR_RATE_INCREASED"
    APPLICATION_SLOWDOWN = "APPLICATION_SLOWDOWN"
    APPLICATION_UNEXPECTED_HIGH_LOAD = "APPLICATION_UNEXPECTED_HIGH_LOAD"
    APPLICATION_UNEXPECTED_LOW_LOAD = "APPLICATION_UNEXPECTED_LOW_LOAD"
    AWS_LAMBDA_HIGH_ERROR_RATE = "AWS_LAMBDA_HIGH_ERROR_RATE"
    CUSTOM_APPLICATION_ERROR_RATE_INCREASED = "CUSTOM_APPLICATION_ERROR_RATE_INCREASED"
    CUSTOM_APPLICATION_SLOWDOWN = "CUSTOM_APPLICATION_SLOWDOWN"
    CUSTOM_APPLICATION_UNEXPECTED_HIGH_LOAD = "CUSTOM_APPLICATION_UNEXPECTED_HIGH_LOAD"
    CUSTOM_APPLICATION_UNEXPECTED_LOW_LOAD = "CUSTOM_APPLICATION_UNEXPECTED_LOW_LOAD"
    CUSTOM_APP_CRASH_RATE_INCREASED = "CUSTOM_APP_CRASH_RATE_INCREASED"
    DATABASE_CONNECTION_FAILURE = "DATABASE_CONNECTION_FAILURE"
    DATA_CENTER_SERVICE_PERFORMANCE_DEGRADATION = "DATA_CENTER_SERVICE_PERFORMANCE_DEGRADATION"
    DATA_CENTER_SERVICE_UNAVAILABLE = "DATA_CENTER_SERVICE_UNAVAILABLE"
    EBS_VOLUME_HIGH_LATENCY = "EBS_VOLUME_HIGH_LATENCY"
    EC2_HIGH_CPU = "EC2_HIGH_CPU"
    ELB_HIGH_BACKEND_ERROR_RATE = "ELB_HIGH_BACKEND_ERROR_RATE"
    ENTERPRICE_APPLICATION_PERFORMANCE_DEGRADATION = "ENTERPRICE_APPLICATION_PERFORMANCE_DEGRADATION"
    ENTERPRISE_APPLICATION_UNAVAILABLE = "ENTERPRISE_APPLICATION_UNAVAILABLE"
    ESXI_GUEST_ACTIVE_SWAP_WAIT = "ESXI_GUEST_ACTIVE_SWAP_WAIT"
    ESXI_GUEST_CPU_LIMIT_REACHED = "ESXI_GUEST_CPU_LIMIT_REACHED"
    ESXI_HOST_CPU_SATURATION = "ESXI_HOST_CPU_SATURATION"
    ESXI_HOST_DATASTORE_LOW_DISK_SPACE = "ESXI_HOST_DATASTORE_LOW_DISK_SPACE"
    ESXI_HOST_DISK_QUEUE_SLOW = "ESXI_HOST_DISK_QUEUE_SLOW"
    ESXI_HOST_DISK_SLOW = "ESXI_HOST_DISK_SLOW"
    ESXI_HOST_MEMORY_SATURATION = "ESXI_HOST_MEMORY_SATURATION"
    ESXI_HOST_NETWORK_PROBLEMS = "ESXI_HOST_NETWORK_PROBLEMS"
    ESXI_HOST_OVERLOADED_STORAGE = "ESXI_HOST_OVERLOADED_STORAGE"
    ESXI_VM_IMPACT_HOST_CPU_SATURATION = "ESXI_VM_IMPACT_HOST_CPU_SATURATION"
    ESXI_VM_IMPACT_HOST_MEMORY_SATURATION = "ESXI_VM_IMPACT_HOST_MEMORY_SATURATION"
    EXTERNAL_SYNTHETIC_TEST_OUTAGE = "EXTERNAL_SYNTHETIC_TEST_OUTAGE"
    EXTERNAL_SYNTHETIC_TEST_SLOWDOWN = "EXTERNAL_SYNTHETIC_TEST_SLOWDOWN"
    HOST_OF_SERVICE_UNAVAILABLE = "HOST_OF_SERVICE_UNAVAILABLE"
    HTTP_CHECK_GLOBAL_OUTAGE = "HTTP_CHECK_GLOBAL_OUTAGE"
    HTTP_CHECK_LOCAL_OUTAGE = "HTTP_CHECK_LOCAL_OUTAGE"
    HTTP_CHECK_TEST_LOCATION_SLOWDOWN = "HTTP_CHECK_TEST_LOCATION_SLOWDOWN"
    MOBILE_APPLICATION_ERROR_RATE_INCREASED = "MOBILE_APPLICATION_ERROR_RATE_INCREASED"
    MOBILE_APPLICATION_SLOWDOWN = "MOBILE_APPLICATION_SLOWDOWN"
    MOBILE_APPLICATION_UNEXPECTED_HIGH_LOAD = "MOBILE_APPLICATION_UNEXPECTED_HIGH_LOAD"
    MOBILE_APPLICATION_UNEXPECTED_LOW_LOAD = "MOBILE_APPLICATION_UNEXPECTED_LOW_LOAD"
    MOBILE_APP_CRASH_RATE_INCREASED = "MOBILE_APP_CRASH_RATE_INCREASED"
    MONITORING_UNAVAILABLE = "MONITORING_UNAVAILABLE"
    OSI_DISK_LOW_INODES = "OSI_DISK_LOW_INODES"
    OSI_GRACEFULLY_SHUTDOWN = "OSI_GRACEFULLY_SHUTDOWN"
    OSI_HIGH_CPU = "OSI_HIGH_CPU"
    OSI_HIGH_MEMORY = "OSI_HIGH_MEMORY"
    OSI_LOW_DISK_SPACE = "OSI_LOW_DISK_SPACE"
    OSI_NIC_DROPPED_PACKETS_HIGH = "OSI_NIC_DROPPED_PACKETS_HIGH"
    OSI_NIC_ERRORS_HIGH = "OSI_NIC_ERRORS_HIGH"
    OSI_NIC_UTILIZATION_HIGH = "OSI_NIC_UTILIZATION_HIGH"
    OSI_SLOW_DISK = "OSI_SLOW_DISK"
    OSI_UNEXPECTEDLY_UNAVAILABLE = "OSI_UNEXPECTEDLY_UNAVAILABLE"
    PGI_OF_SERVICE_UNAVAILABLE = "PGI_OF_SERVICE_UNAVAILABLE"
    PGI_UNAVAILABLE = "PGI_UNAVAILABLE"
    PG_LOW_INSTANCE_COUNT = "PG_LOW_INSTANCE_COUNT"
    PROCESS_CRASHED = "PROCESS_CRASHED"
    PROCESS_HIGH_GC_ACTIVITY = "PROCESS_HIGH_GC_ACTIVITY"
    PROCESS_MEMORY_RESOURCE_EXHAUSTED = "PROCESS_MEMORY_RESOURCE_EXHAUSTED"
    PROCESS_NA_HIGH_CONN_FAIL_RATE = "PROCESS_NA_HIGH_CONN_FAIL_RATE"
    PROCESS_NA_HIGH_LOSS_RATE = "PROCESS_NA_HIGH_LOSS_RATE"
    PROCESS_THREADS_RESOURCE_EXHAUSTED = "PROCESS_THREADS_RESOURCE_EXHAUSTED"
    RDS_HIGH_CPU = "RDS_HIGH_CPU"
    RDS_HIGH_LATENCY = "RDS_HIGH_LATENCY"
    RDS_LOW_MEMORY = "RDS_LOW_MEMORY"
    RDS_LOW_STORAGE_SPACE = "RDS_LOW_STORAGE_SPACE"
    RDS_OF_SERVICE_UNAVAILABLE = "RDS_OF_SERVICE_UNAVAILABLE"
    RDS_RESTART_SEQUENCE = "RDS_RESTART_SEQUENCE"
    SERVICE_ERROR_RATE_INCREASED = "SERVICE_ERROR_RATE_INCREASED"
    SERVICE_SLOWDOWN = "SERVICE_SLOWDOWN"
    SERVICE_UNEXPECTED_HIGH_LOAD = "SERVICE_UNEXPECTED_HIGH_LOAD"
    SERVICE_UNEXPECTED_LOW_LOAD = "SERVICE_UNEXPECTED_LOW_LOAD"
    SYNTHETIC_GLOBAL_OUTAGE = "SYNTHETIC_GLOBAL_OUTAGE"
    SYNTHETIC_LOCAL_OUTAGE = "SYNTHETIC_LOCAL_OUTAGE"
    SYNTHETIC_NODE_OUTAGE = "SYNTHETIC_NODE_OUTAGE"
    SYNTHETIC_PRIVATE_LOCATION_OUTAGE = "SYNTHETIC_PRIVATE_LOCATION_OUTAGE"
    SYNTHETIC_TEST_LOCATION_SLOWDOWN = "SYNTHETIC_TEST_LOCATION_SLOWDOWN"

    def __str__(self) -> str:
        return self.value


class SeverityLevel(Enum):
    AVAILABILITY = "AVAILABILITY"
    CUSTOM_ALERT = "CUSTOM_ALERT"
    ERROR = "ERROR"
    MONITORING_UNAVAILABLE = "MONITORING_UNAVAILABLE"
    PERFORMANCE = "PERFORMANCE"
    RESOURCE_CONTENTION = "RESOURCE_CONTENTION"

    def __str__(self) -> str:
        return self.value



from dynatrace import Dynatrace
from dynatrace.configuration_v1.metric_events import (
    AggregationType,
    DisabledReason,
    MetricEventAlertingScopeFilterType,
    MetricEventAlertingScope,
    MetricEventAutoAdaptiveBaselineMonitoringStrategy,
    MetricEventDimension,
    MetricEventDimensionsFilterType,
    MetricEventMonitoringStrategyType,
    MetricEventShortRepresentation,
    MetricEventStaticThresholdMonitoringStrategy,
    Severity,
    WarningReason,
    Unit,
)
from dynatrace.pagination import PaginatedList

from typing import List

STATIC_ID = "ruxit.python.rabbitmq:node_status:node_failed"
STATIC_NAME = "RabbitMQ Node failed"
BASELINE_ID = "d3baaaed-3441-4931-bf24-25c4e12e137f"
BASELINE_NAME = "Mint alert for static"


def test_list(dt: Dynatrace):
    metric_events = dt.anomaly_detection_metric_events.list()
    assert isinstance(metric_events, PaginatedList)

    list_metric_events = list(metric_events)
    assert len(list_metric_events) == 193

    first = list_metric_events[0]
    assert isinstance(first, MetricEventShortRepresentation)

    assert first.id == STATIC_ID
    assert first.name == STATIC_NAME


def test_get_full_configuration(dt: Dynatrace):
    metric_events = dt.anomaly_detection_metric_events.list()
    list_metric_events = list(metric_events)

    for metric_event in list_metric_events:
        if metric_event.id == STATIC_ID:
            # static
            full = metric_event.get_full_metric_event()

            # type checks
            assert isinstance(full.name, str)
            assert isinstance(full.metric_id, str)
            assert isinstance(full.severity, Severity)
            assert isinstance(full.enabled, bool)
            assert isinstance(full.disabled_reason, DisabledReason)
            assert isinstance(full.aggregation_type, AggregationType)
            assert isinstance(full.warning_reason, WarningReason)
            assert isinstance(full.alerting_scope, List)
            assert isinstance(full.monitoring_strategy, MetricEventStaticThresholdMonitoringStrategy)
            assert isinstance(full.metric_dimensions, List)
            assert isinstance(full.primary_dimension_key, type(None))

            # value checks
            assert full.name == STATIC_NAME
            assert full.id == STATIC_ID
            assert full.aggregation_type == AggregationType.VALUE
            assert full.severity == Severity.AVAILABILITY
            assert full.enabled == True
            assert full.disabled_reason == DisabledReason.NONE
            assert full.monitoring_strategy.type == MetricEventMonitoringStrategyType.STATIC_THRESHOLD
            assert full.monitoring_strategy.unit == Unit.COUNT
            assert full.monitoring_strategy.violating_samples == 3
            assert full.metric_dimensions[0].filter_type == MetricEventDimensionsFilterType.STRING

        elif metric_event.id == BASELINE_ID:
            # static
            full = metric_event.get_full_metric_event()

            # type checks
            assert isinstance(full.monitoring_strategy, MetricEventAutoAdaptiveBaselineMonitoringStrategy)

            # value checks
            assert full.monitoring_strategy.type == MetricEventMonitoringStrategyType.AUTO_ADAPTIVE_BASELINE

            break

        else:
            continue



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from datetime import datetime
from enum import Enum
from pathlib import Path
from typing import List, Dict, Any, Optional, Union

from requests import Response

from dynatrace.environment_v2.schemas import ConfigurationMetadata
from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.environment_v2.monitored_entities import EntityShortRepresentation
from dynatrace.http_client import HttpClient
from dynatrace.pagination import PaginatedList


class ExtensionService:

    # TODO - Early adopter as of May 12th 2021. Check back for updates
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client
        pass

    def list(self, page_size: int = 200) -> PaginatedList["ExtensionDto"]:
        """
        List all uploaded extensions

        :param page_size: The number of results per result page. Must be between 1 and 500
            Default value : 200
        """
        params = {"pageSize": page_size}
        return PaginatedList(ExtensionDto, self.__http_client, f"/api/config/v1/extensions", params, list_item="extensions")

    def get(self, extension_id: str):
        response = self.__http_client.make_request(f"/api/config/v1/extensions/{extension_id}").json()
        return Extension(raw_element=response)

    def post(self, zip_file_path: str) -> EntityShortRepresentation:

        file = Path(zip_file_path)
        with open(file, "rb") as f:
            response = self.__http_client.make_request("/api/config/v1/extensions", method="POST", files={"file": f})

        return EntityShortRepresentation(raw_element=response.json())

    def validate(self, zip_file_path: str) -> Response:

        file = Path(zip_file_path)
        with open(file, "rb") as f:
            return self.__http_client.make_request("/api/config/v1/extensions/validator", method="POST", files={"file": f})

    def list_instances(self, extension_id: str, page_size: int = 200) -> PaginatedList["ExtensionShortRepresentation"]:
        params = {"pageSize": page_size}
        return PaginatedList(
            ExtensionShortRepresentation,
            self.__http_client,
            f"/api/config/v1/extensions/{extension_id}/instances",
            list_item="configurationsList",
            target_params=params,
        )

    def get_instance(self, extension_id: str, configuration_id: str):
        response = self.__http_client.make_request(f"/api/config/v1/extensions/{extension_id}/instances/{configuration_id}").json()
        return ExtensionConfigurationDto(http_client=self.__http_client, raw_element=response)

    def post_instance(self, extension_configuration: "ExtensionConfigurationDto"):
        return extension_configuration.post()

    def validate_instance(self, extension_configuration: "ExtensionConfigurationDto") -> Response:
        return extension_configuration.validate()

    def get_instance_configuration(self, extension_id: str, configuration_id: str) -> "ExtensionConfigurationDto":
        response = self.__http_client.make_request(f"/api/config/v1/extensions/{extension_id}/instances/{configuration_id}").json()
        return ExtensionConfigurationDto(http_client=self.__http_client, raw_element=response)

    # TODO - Can't implement put_instance because of APM-300262, fixed in 1.219

    def delete_instance_configuration(self, extension_id: str, configuration_id: str) -> Response:
        return self.__http_client.make_request(f"/api/config/v1/extensions/{extension_id}/instances/{configuration_id}", method="DELETE")

    def get_global_configuration(self, extension_id: str) -> "GlobalExtensionConfiguration":
        response = self.__http_client.make_request(f"/api/config/v1/extensions/{extension_id}/global").json()
        return GlobalExtensionConfiguration(raw_element=response)

    def get_binary(self, extension_id: str) -> bytes:
        return self.__http_client.make_request(f"/api/config/v1/extensions/{extension_id}/binary").content

    def list_states(self, extension_id: str) -> PaginatedList["ExtensionState"]:
        return PaginatedList(
            ExtensionState,
            self.__http_client,
            f"/api/config/v1/extensions/{extension_id}/states",
            list_item="states",
        )

    def list_activegate_extension_modules(self) -> PaginatedList[EntityShortRepresentation]:
        return PaginatedList(
            EntityShortRepresentation,
            self.__http_client,
            f"/api/config/v1/extensions/activeGateExtensionModules",
            list_item="values",
        )

    def create_instance(
        self,
        extension_id: str,
        properties: Dict[str, Any] = None,
        enabled: Optional[bool] = True,
        use_global: Optional[bool] = True,
        host_id: Optional[str] = None,
        activegate: Optional[EntityShortRepresentation] = None,
        endpoint_id: Optional[str] = None,
        endpoint_name: Optional[str] = None,
    ) -> "ExtensionConfigurationDto":

        raw_element = {
            "extensionId": extension_id,
            "enabled": enabled,
            "useGlobal": use_global,
            "properties": properties,
            "hostId": host_id,
            "activeGate": activegate._raw_element if activegate else {},
            "endpointId": endpoint_id,
            "endpointName": endpoint_name,
        }

        return ExtensionConfigurationDto(http_client=self.__http_client, raw_element=raw_element)


class ExtensionProperty(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.key: str = raw_element.get("key")
        self.type: str = raw_element.get("type")
        self.default_value: str = raw_element.get("defaultValue")
        self.dropdown_values: List[str] = raw_element.get("dropdownValues")


class ExtensionConfigurationDto(DynatraceObject):
    def post(self):

        return self._http_client.make_request(f"/api/config/v1/extensions/{self.extension_id}/instances", params=self.to_json(), method="POST")

    def validate(self):
        return self._http_client.make_request(f"/api/config/v1/extensions/{self.extension_id}/instances/validator", params=self.to_json(), method="POST")

    def to_json(self) -> Dict[str, Any]:
        return {
            "extensionId": self.extension_id,
            "enabled": self.enabled,
            "useGlobal": self.use_global,
            "properties": self.properties,
            "hostId": self.host_id,
            "activeGate": self.active_gate.to_json(),
            "endpointId": self.endpoint_id,
            "endpointName": self.endpoint_name,
        }

    def _create_from_raw_data(self, raw_element):
        self.extension_id: str = raw_element.get("extensionId")
        self.enabled: bool = raw_element.get("enabled")
        self.use_global: bool = raw_element.get("useGlobal", True)
        self.properties: Dict[str, Any] = raw_element.get("properties")
        self.host_id: str = raw_element.get("hostId")
        self.active_gate: EntityShortRepresentation = EntityShortRepresentation(self._http_client, None, raw_element.get("activeGate"))
        self.endpoint_id: str = raw_element.get("endpointId")
        self.endpoint_name: str = raw_element.get("endpointName")


class ExtensionShortRepresentation(EntityShortRepresentation):
    def get_full_configuration(self, extension_id: str):
        """
        Gets the full extension configuration for this ExtensionShortRepresentation
        """
        response = self._http_client.make_request(f"/api/config/v1/extensions/{extension_id}/instances/{self.id}").json()
        return ExtensionConfigurationDto(self._http_client, raw_element=response)


class Extension(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.id: str = raw_element.get("id")
        self.name: str = raw_element.get("name")
        self.type: str = ExtensionType(raw_element.get("type"))
        self.version: str = raw_element.get("version")
        self.metric_group: str = raw_element.get("metricGroup")
        self.metadata: ConfigurationMetadata = ConfigurationMetadata(self._http_client, None, raw_element.get("metadata"))
        self.properties: List[ExtensionProperty] = [ExtensionProperty(self._http_client, None, prop) for prop in raw_element.get("properties")]


class ExtensionType(Enum):
    ACTIVEGATE = "ACTIVEGATE"
    CODEMODULE = "CODEMODULE"
    JMX = "JMX"
    ONEAGENT = "ONEAGENT"
    PMI = "PMI"
    UNKNOWN = "UNKNOWN"


class ExtensionDto(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.id: str = raw_element.get("id")
        self.name: str = raw_element.get("name")
        self.type: ExtensionType = ExtensionType(raw_element.get("type"))

    def get_full_extension(self) -> Extension:
        """
        Gets the full extension for this ExtensionDto
        """
        response = self._http_client.make_request(f"/api/config/v1/extensions/{self.id}").json()
        return Extension(self._http_client, None, response)

    @property
    def instances(self, page_size: int = 200) -> PaginatedList["ExtensionShortRepresentation"]:
        """
        Returns the list of instances for this extension
        :param page_size: Page size, default 200
        """
        params = {"pageSize": page_size}
        return PaginatedList(
            ExtensionShortRepresentation,
            self._http_client,
            f"/api/config/v1/extensions/{self.id}/instances",
            list_item="configurationsList",
            target_params=params,
        )

    def delete(self) -> Response:
        """
        Deletes the ZIP file of this extension
        """
        return self._http_client.make_request(f"/api/config/v1/extensions/{self.id}", method="DELETE")


class GlobalExtensionConfiguration(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.enabled: bool = raw_element.get("enabled")
        self.extension_id: Optional[str] = raw_element.get("extensionId")
        self.infraOnlyEnabled: Optional[bool] = raw_element.get("infraOnlyEnabled")
        self.properties: Optional[Dict[str, Any]] = raw_element.get("properties")


class ExtensionStateEnum(Enum):
    ERROR_AUTH = "ERROR_AUTH"
    ERROR_COMMUNICATION_FAILURE = "ERROR_COMMUNICATION_FAILURE"
    ERROR_CONFIG = "ERROR_CONFIG"
    ERROR_TIMEOUT = "ERROR_TIMEOUT"
    ERROR_UNKNOWN = "ERROR_UNKNOWN"
    INCOMPATIBLE = "INCOMPATIBLE"
    LIMIT_REACHED = "LIMIT_REACHED"
    NOTHING_TO_REPORT = "NOTHING_TO_REPORT"
    OK = "OK"
    STATE_TYPE_UNKNOWN = "STATE_TYPE_UNKNOWN"
    UNINITIALIZED = "UNINITIALIZED"
    UNSUPPORTED = "UNSUPPORTED"
    WAITING_FOR_STATE = "WAITING_FOR_STATE"


class ExtensionState(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.extension_id: Optional[str] = raw_element.get("extensionId")
        self.version: Optional[str] = raw_element.get("version")
        self.endpoint_id: Optional[str] = raw_element.get("endpointId")
        self.state: Optional[ExtensionStateEnum] = ExtensionStateEnum(raw_element.get("state"))
        self.state_description: Optional[str] = raw_element.get("stateDescription")
        self.host_id: Optional[str] = raw_element.get("hostId")
        self.process_id: Optional[str] = raw_element.get("processId")
        self.timestamp: Optional[datetime] = datetime.utcfromtimestamp(raw_element.get("timestamp") / 1000)



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from datetime import datetime
from enum import Enum
from typing import Optional, Union, Dict, Any, List

from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient
from dynatrace.pagination import PaginatedList
from dynatrace.utils import timestamp_to_string


class AuditLogsService:

    # TODO - Early adopter as of May 14th 2021, check back later for changes
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(
        self,
        log_filter: Optional[str] = None,
        time_from: Optional[Union[datetime, str]] = None,
        time_to: Optional[Union[datetime, str]] = None,
        sort: Optional[str] = None,
    ) -> PaginatedList["AuditLogEntry"]:
        params = {"filter": log_filter, "from": timestamp_to_string(time_from), "to": timestamp_to_string(time_to), "sort": sort}
        return PaginatedList(
            target_class=AuditLogEntry, http_client=self.__http_client, target_url="/api/v2/auditlogs", target_params=params, list_item="auditLogs"
        )

    def get(self, log_id: str) -> "AuditLogEntry":
        response = self.__http_client.make_request(f"/api/v2/auditlogs/{log_id}").json()
        return AuditLogEntry(raw_element=response)


class EventType(Enum):
    CREATE = "CREATE"
    DELETE = "DELETE"
    GENERAL = "GENERAL"
    GET = "GET"
    LOGIN = "LOGIN"
    LOGOUT = "LOGOUT"
    PATCH = "PATCH"
    POST = "POST"
    PUT = "PUT"
    READ = "READ"
    REVOKE = "REVOKE"
    UPDATE = "UPDATE"


class Category(Enum):
    CONFIG = "CONFIG"
    DEBUG_UI = "DEBUG_UI"
    REST = "REST"
    TOKEN = "TOKEN"
    WEB_UI = "WEB_UI"


class UserType(Enum):
    PUBLIC_TOKEN_IDENTIFIER = "PUBLIC_TOKEN_IDENTIFIER"
    REQUEST_ID = "REQUEST_ID"
    SERVICE_NAME = "SERVICE_NAME"
    TOKEN_HASH = "TOKEN_HASH"
    USER_NAME = "USER_NAME"


class AuditLogEntry(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.category: Category = Category(raw_element.get("category"))
        self.environment_id: str = raw_element.get("environmentId")
        self.event_type: EventType = EventType(raw_element.get("eventType"))
        self.log_id: str = raw_element.get("logId")
        self.success: bool = raw_element.get("success")
        self.timestamp: datetime = datetime.utcfromtimestamp(raw_element.get("timestamp") / 1000)
        self.user: str = raw_element.get("user")
        self.user_type: UserType = UserType(raw_element.get("userType"))

        self.entity_id: Optional[str] = raw_element.get("entityId")
        self.user_origin: Optional[str] = raw_element.get("userOrigin")
        self.message: Optional[str] = raw_element.get("message")
        self.patch: Optional[List[Dict[str, Any]]] = raw_element.get("patch")



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging
from typing import Dict, Optional

from dynatrace.configuration_v1.alerting_profiles import AlertingProfileService
from dynatrace.configuration_v1.anomaly_detection_process_groups import AnomalyDetectionPGService
from dynatrace.configuration_v1.auto_tags import AutoTagService
from dynatrace.configuration_v1.dashboard import DashboardService
from dynatrace.configuration_v1.extensions import ExtensionService
from dynatrace.configuration_v1.maintenance_windows import MaintenanceWindowService
from dynatrace.configuration_v1.metric_events import MetricEventService
from dynatrace.configuration_v1.notifications import NotificationService
from dynatrace.configuration_v1.plugins import PluginService
from dynatrace.configuration_v1.oneagent_on_a_host import OneAgentOnAHostService as OneAgentOnAHostConfigService
from dynatrace.configuration_v1.oneagent_in_a_hostgroup import OneAgentInAHostGroupService
from dynatrace.configuration_v1.oneagent_environment_wide_configuration import OneAgentEnvironmentWideConfigService
from dynatrace.environment_v1.cluster_time import ClusterTimeService
from dynatrace.environment_v1.custom_device import CustomDeviceService
from dynatrace.environment_v1.event import EventService
from dynatrace.environment_v1.oneagents import OneAgentOnAHostService
from dynatrace.environment_v1.smartscape_hosts import SmartScapeHostsService
from dynatrace.environment_v1.synthetic_monitors import SyntheticMonitorsService
from dynatrace.environment_v1.synthetic_third_party import ThirdPartySyntheticTestsService
from dynatrace.environment_v1.timeseries import TimeSerieService
from dynatrace.environment_v1.deployment import DeploymentService
from dynatrace.environment_v2.activegates import ActiveGateService
from dynatrace.environment_v2.activegates_autoupdate_configuration import ActiveGateAutoUpdateConfigurationService
from dynatrace.environment_v2.activegates_autoupdate_jobs import ActiveGateAutoUpdateJobsService
from dynatrace.environment_v2.audit_logs import AuditLogsService
from dynatrace.environment_v2.extensions import ExtensionsServiceV2
from dynatrace.environment_v2.events import EventServiceV2
from dynatrace.environment_v2.monitored_entities import EntityService
from dynatrace.environment_v2.custom_tags import CustomTagService
from dynatrace.environment_v2.metrics import MetricService
from dynatrace.environment_v2.networkzones import NetworkZoneService
from dynatrace.environment_v2.tokens_api import TokenService
from dynatrace.environment_v2.tokens_tenant import TenantTokenService
from dynatrace.environment_v2.problems import ProblemService
from dynatrace.environment_v2.service_level_objectives import SloService
from dynatrace.environment_v2.logs import LogService
from dynatrace.http_client import HttpClient


class Dynatrace:
    def __init__(
        self,
        base_url: str,
        token: str,
        log: logging.Logger = None,
        proxies: Dict = None,
        too_many_requests_strategy=None,
        retries: int = 0,
        retry_delay_ms: int = 0,
        mc_jsession_id: Optional[str] = None,
        mc_b925d32c: Optional[str] = None,
        mc_sso_csrf_cookie: Optional[str] = None,
    ):
        self.__http_client = HttpClient(
            base_url, token, log, proxies, too_many_requests_strategy, retries, retry_delay_ms, mc_jsession_id, mc_b925d32c, mc_sso_csrf_cookie,
        )

        self.activegates: ActiveGateService = ActiveGateService(self.__http_client)
        self.activegates_autoupdate_configuration: ActiveGateAutoUpdateConfigurationService = ActiveGateAutoUpdateConfigurationService(self.__http_client)
        self.activegates_autoupdate_jobs = ActiveGateAutoUpdateJobsService(self.__http_client)
        self.alerting_profiles: AlertingProfileService = AlertingProfileService(self.__http_client)
        self.anomaly_detection_metric_events = MetricEventService(self.__http_client)
        self.anomaly_detection_process_groups = AnomalyDetectionPGService(self.__http_client)
        self.audit_logs: AuditLogsService = AuditLogsService(self.__http_client)
        self.auto_tags: AutoTagService = AutoTagService(self.__http_client)
        self.cluster_time: ClusterTimeService = ClusterTimeService(self.__http_client)
        self.custom_devices: CustomDeviceService = CustomDeviceService(self.__http_client)
        self.custom_tags: CustomTagService = CustomTagService(self.__http_client)
        self.dashboards: DashboardService = DashboardService(self.__http_client)
        self.deployment: DeploymentService = DeploymentService(self.__http_client)
        self.entities: EntityService = EntityService(self.__http_client)
        self.events: EventService = EventService(self.__http_client)
        self.events_v2: EventServiceV2 = EventServiceV2(self.__http_client)
        self.extensions: ExtensionService = ExtensionService(self.__http_client)
        self.extensions_v2: ExtensionsServiceV2 = ExtensionsServiceV2(self.__http_client)
        self.logs: LogService = LogService(self.__http_client)
        self.maintenance_windows = MaintenanceWindowService(self.__http_client)
        self.metrics: MetricService = MetricService(self.__http_client)
        self.network_zones: NetworkZoneService = NetworkZoneService(self.__http_client)
        self.notifications: NotificationService = NotificationService(self.__http_client)
        self.oneagents: OneAgentOnAHostService = OneAgentOnAHostService(self.__http_client)
        self.oneagents_config_environment: OneAgentEnvironmentWideConfigService = OneAgentEnvironmentWideConfigService(self.__http_client)
        self.oneagents_config_host: OneAgentOnAHostConfigService = OneAgentOnAHostConfigService(self.__http_client)
        self.oneagents_config_hostgroup: OneAgentInAHostGroupService = OneAgentInAHostGroupService(self.__http_client)
        self.plugins: PluginService = PluginService(self.__http_client)
        self.problems: ProblemService = ProblemService(self.__http_client)
        self.slos: SloService = SloService(self.__http_client)
        self.smartscape_hosts: SmartScapeHostsService = SmartScapeHostsService(self.__http_client)
        self.synthetic_monitors: SyntheticMonitorsService = SyntheticMonitorsService(self.__http_client)
        self.tenant_tokens = TenantTokenService(self.__http_client)
        self.third_part_synthetic_tests: ThirdPartySyntheticTestsService = ThirdPartySyntheticTestsService(self.__http_client)
        self.timeseries: TimeSerieService = TimeSerieService(self.__http_client)
        self.tokens: TokenService = TokenService(self.__http_client)



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from typing import Dict, Any, Optional
from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient


class TenantTokenService:
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def cancel_rotation(self) -> "TenantTokenConfig":
        return TenantTokenConfig(raw_element=self.__http_client.make_request("/api/v2/tenantTokenRotation/cancel", method="POST").json())

    def start_rotation(self) -> "TenantTokenConfig":
        return TenantTokenConfig(raw_element=self.__http_client.make_request("/api/v2/tenantTokenRotation/start", method="POST").json())

    def finish_rotation(self) -> "TenantTokenConfig":
        return TenantTokenConfig(raw_element=self.__http_client.make_request("/api/v2/tenantTokenRotation/finish", method="POST").json())


class TenantTokenConfig(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.active: Optional[TenantToken] = TenantToken(raw_element=raw_element.get("active"))
        self.old: Optional[TenantToken] = TenantToken(raw_element=raw_element.get("old"))


class TenantToken(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.value: str = raw_element.get("value")



from dynatrace import Dynatrace
from dynatrace.configuration_v1.oneagent_on_a_host import (
    HostConfig,
    HostAutoUpdateConfig,
    MonitoringConfig,
    AutoUpdateSetting,
    TechMonitoringList,
    EffectiveSetting,
    MonitoringMode,
)
from dynatrace.configuration_v1.schemas import UpdateWindowsConfig, UpdateWindow, ConfigurationMetadata, Technology, TechnologyType, SettingScope

HOST_ID = "HOST-abcd123457"
CLUSTER_VERSION = "1.222.47.20210712-162143"


def test_get(dt: Dynatrace):
    oa_host_config = dt.oneagents_config_host.get(HOST_ID)

    # type checks
    assert isinstance(oa_host_config, HostConfig)
    assert isinstance(oa_host_config.id, str)
    assert isinstance(oa_host_config.auto_update_config, HostAutoUpdateConfig)
    assert isinstance(oa_host_config.monitoring_config, MonitoringConfig)
    assert isinstance(oa_host_config.tech_monitoring_config_list, TechMonitoringList)

    # value checks
    assert oa_host_config.id == HOST_ID


def test_get_autoupdate(dt: Dynatrace):
    oa_autoupdate = dt.oneagents_config_host.get_autoupdate(HOST_ID)

    # type checks
    assert isinstance(oa_autoupdate, HostAutoUpdateConfig)
    assert isinstance(oa_autoupdate.id, str)
    assert isinstance(oa_autoupdate.setting, AutoUpdateSetting)
    assert isinstance(oa_autoupdate.version, (str, type(None)))
    assert isinstance(oa_autoupdate.effective_setting, EffectiveSetting)
    assert isinstance(oa_autoupdate.effective_version, (str, type(None)))
    assert isinstance(oa_autoupdate.update_windows, UpdateWindowsConfig)
    assert isinstance(oa_autoupdate.metadata, ConfigurationMetadata)
    assert all(isinstance(uw, UpdateWindow) for uw in oa_autoupdate.update_windows.windows)

    # value checks
    assert oa_autoupdate.id == HOST_ID
    assert oa_autoupdate.setting == AutoUpdateSetting.DISABLED
    assert oa_autoupdate.version is None
    assert oa_autoupdate.effective_setting == EffectiveSetting.DISABLED
    assert oa_autoupdate.effective_version is None
    assert oa_autoupdate.update_windows.windows == []
    assert oa_autoupdate.metadata.cluster_version == CLUSTER_VERSION


def test_get_monitoring(dt: Dynatrace):
    oa_monitoring = dt.oneagents_config_host.get_monitoring(HOST_ID)

    # type checks
    assert isinstance(oa_monitoring, MonitoringConfig)
    assert isinstance(oa_monitoring.id, str)
    assert isinstance(oa_monitoring.monitoring_enabled, bool)
    assert isinstance(oa_monitoring.monitoring_mode, MonitoringMode)
    assert isinstance(oa_monitoring.metadata, ConfigurationMetadata)

    # value checks
    assert oa_monitoring.id == HOST_ID
    assert oa_monitoring.monitoring_enabled == True
    assert oa_monitoring.monitoring_mode == MonitoringMode.FULL_STACK
    assert oa_monitoring.metadata.cluster_version == CLUSTER_VERSION


def test_get_technologies(dt: Dynatrace):
    oa_technologies = dt.oneagents_config_host.get_technologies(HOST_ID)

    # type checks
    assert isinstance(oa_technologies.metadata, ConfigurationMetadata)
    assert isinstance(oa_technologies, TechMonitoringList)
    assert all(isinstance(t, Technology) for t in oa_technologies.technologies)
    for tech in oa_technologies.technologies:
        assert isinstance(tech.type, TechnologyType)
        assert isinstance(tech.monitoring_enabled, bool)
        assert isinstance(tech.scope, (SettingScope, type(None)))

    # value checks
    assert len(oa_technologies.technologies) == 4
    assert oa_technologies.technologies[0].type == TechnologyType.LOG_ANALYTICS
    assert oa_technologies.technologies[0].monitoring_enabled == True
    assert oa_technologies.technologies[0].scope == SettingScope.ENVIRONMENT
    assert oa_technologies.metadata.cluster_version == CLUSTER_VERSION



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from typing import Union, Optional, List
from datetime import datetime

from requests import Response

from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient
from dynatrace.pagination import PaginatedList
from dynatrace.utils import timestamp_to_string, iso8601_to_datetime


SCOPE_INSTALLER_DOWNLOAD = "InstallerDownload"  # PaaS integration - Installer download.
SCOPE_DATA_EXPORT = "DataExport"  # Access problem and event feed, metrics, and topology.
SCOPE_PLUGIN_UPLOAD = "PluginUpload"  # Upload Extension.
SCOPE_SUPPORT_ALERT = "SupportAlert"  # PaaS integration - Support alert.
SCOPE_DCRUM_INTEGRATION = "DcrumIntegration"  # Dynatrace module integration - NAM.
SCOPE_ADVANCED_SYNTHETIC_INTEGRATION = "AdvancedSyntheticIntegration"  # Dynatrace module integration - Synthetic Classic.
SCOPE_EXTERNAL_SYNTHETIC_INTEGRATION = "ExternalSyntheticIntegration"  # Create and read synthetic monitors, locations, and nodes.
SCOPE_APP_MON_INTEGRATION = "AppMonIntegration"  # Dynatrace module integration - AppMon.
SCOPE_LOG_EXPORT = "LogExport"  # Read log content.
SCOPE_READ_CONFIG = "ReadConfig"  # Read configuration.
SCOPE_WRITE_CONFIG = "WriteConfig"  # Write configuration.
SCOPE_DTAQLAccess = "DTAQLAccess"  # User sessions.
SCOPE_USER_SESSION_ANONYMIZATION = "UserSessionAnonymization"  # Anonymize user session data for data privacy reasons.
SCOPE_DATA_PRIVACY = "DataPrivacy"  # Change data privacy settings.
SCOPE_CAPTURE_REQUEST_DATA = "CaptureRequestData"  # Capture request data.
SCOPE_DAVIS = "Davis"  # Dynatrace module integration - Davis.
SCOPE_DSS_FILE_MANAGEMENT = "DssFileManagement"  # Mobile symbolication file management.
SCOPE_LOG_IMPORT = "LogImport"  # Log import.
SCOPE_RUM_JAVA_SCRIPT_TAG_MANAGEMENT = "RumJavaScriptTagManagement"  # Real user monitoring JavaScript tag management.
SCOPE_TENANT_TOKEN_MANAGEMENT = "TenantTokenManagement"  # Token management.
SCOPE_ACTIVEGATE_CERT_MANAGEMENT = "ActiveGateCertManagement"  # ActiveGate certificate management.
SCOPE_REST_REQUEST_FORWARDING = "RestRequestForwarding"  # Fetch data from a remote environment.
SCOPE_READ_SYNTHETIC_DATA = "ReadSyntheticData"  # Read synthetic monitors, locations, and nodes.
SCOPE_DATA_IMPORT = "DataImport"  # Data ingest, e.g.  # metrics and events.
SCOPE_AUDIT_LOG_READ = "auditLogs.read"  # Read audit logs.
SCOPE_METRICS_READ = "metrics.read"  # Read metrics.
SCOPE_METRICS_WRITE = "metrics.write"  # Write metrics.
SCOPE_ENTITIES_READ = "entities.read"  # Read entities.
SCOPE_ENTITIES_WRITE = "entities.write"  # Write entities.
SCOPE_PROBLEMS_READ = "problems.read"  # Read problems.
SCOPE_PROBLEMS_WRITE = "problems.write"  # Write problems.
SCOPE_NETWORK_ZONES_READ = "networkZones.read"  # Read network zones.
SCOPE_NETWORK_ZONES_WRITE = "networkZones.write"  # Write network zones.
SCOPE_ACTIVE_GATES_READ = "activeGates.read"  # Read ActiveGates.
SCOPE_ACTIVE_GATES_WRITE = "activeGates.write"  # Write ActiveGates.
SCOPE_CREDENTIAL_VAULT_READ = "credentialVault.read"  # Read credential vault entries.
SCOPE_CREDENTIAL_VAULT_WRITE = "credentialVault.write"  # Write credential vault entries.
SCOPE_EXTENSIONS_READ = "extensions.read"  # Read extensions.
SCOPE_EXTENSIONS_WRITE = "extensions.write"  # Write extensions.
SCOPE_EXTENSION_CONFIGURATIONS_READ = "extensionConfigurations.read"  # Read extension monitoring configurations.
SCOPE_EXTENSION_CONFIGURATIONS_WRITE = "extensionConfigurations.write"  # Write extension monitoring configurations.
SCOPE_EXTENSION_ENVIRONMENT_READ = "extensionEnvironment.read"  # Read extension environment configurations.
SCOPE_EXTENSION_ENVIRONMENT_WRITE = "extensionEnvironment.write"  # Write extension environment configurations.
SCOPE_METRICS_INGEST = "metrics.ingest"  # Ingest metrics.
SCOPE_SECURITY_PROBLEMS_READ = "securityProblems.read"  # Read security problems.
SCOPE_SECURITY_PROBLEMS_WRITE = "securityProblems.write"  # Write security problems.
SCOPE_SYNTHETIC_LOCATIONS_READ = "syntheticLocations.read"  # Read synthetic locations.
SCOPE_SYNTHETIC_LOCATIONS_WRITE = "syntheticLocations.write"  # Write synthetic locations.
SCOPE_TENANT_TOKEN_ROTATION_WRITE = "tenantTokenRotation.write"  # Tenant token rotation.
SCOPE_SLO_READ = "slo.read"  # Read SLO.
SCOPE_SLO_WRITE = "slo.write"  # Write SLO.
SCOPE_RELEASES_READ = "releases.read"  # Read releases.
SCOPE_API_TOKENS_READ = "apiTokens.read"  # Read API tokens.
SCOPE_API_TOKENS_WRITE = "apiTokens.write"  # Write API tokens.


class TokenService:
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(
        self,
        api_token_selector: Optional[str] = None,
        fields: Optional[str] = None,
        time_from: Optional[Union[str, datetime]] = None,
        time_to: Optional[Union[str, datetime]] = None,
        sort: Optional[str] = None,
    ) -> PaginatedList["ApiToken"]:

        params = {
            "apiTokenSelector": api_token_selector,
            "fields": fields,
            "from": timestamp_to_string(time_from),
            "to": timestamp_to_string(time_to),
            "sort": sort,
        }

        return PaginatedList(ApiToken, self.__http_client, "/api/v2/apiTokens", params, list_item="apiTokens")

    def get(self, token_id: str):
        response = self.__http_client.make_request(f"/api/v2/apiTokens/{token_id}", method="GET")
        return ApiToken(raw_element=response.json())

    def delete(self, token_id: str) -> Response:
        return self.__http_client.make_request(f"/api/v2/apiTokens/{token_id}", method="DELETE")

    def create(
        self, name: str, scopes: List[str], personal_access_token: Optional[bool] = None, expiration_date: Optional[Union[datetime, str]] = None
    ) -> "ApiTokenCreated":

        body = {"personalAccessToken": personal_access_token, "expirationDate": expiration_date, "name": name, "scopes": scopes}

        response = self.__http_client.make_request("/api/v2/apiTokens", method="POST", params=body)
        return ApiTokenCreated(raw_element=response.json())

    def lookup(self, token: str):
        response = self.__http_client.make_request("/api/v2/apiTokens/lookup", method="POST", params={"token": token})
        return ApiToken(raw_element=response.json())


class ApiToken(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.last_used_date: Optional[datetime] = iso8601_to_datetime(raw_element.get("lastUsedDate"))
        self.last_used_ip_address: str = raw_element.get("lastUsedIpAddress")
        self.personal_access_token: bool = raw_element.get("personalAccessToken", False)
        self.expiration_date: Optional[datetime] = iso8601_to_datetime(raw_element.get("expirationDate"))
        self.creation_date: Optional[datetime] = iso8601_to_datetime(raw_element.get("creationDate"))
        self.name: str = raw_element.get("name")
        self.id: str = raw_element.get("id")
        self.owner: str = raw_element.get("owner")
        self.scopes: List[str] = raw_element.get("scopes", [])
        self.enabled: bool = raw_element.get("enabled")

    def delete(self):
        return self._http_client.make_request(f"/api/v2/apiTokens/{self.id}", method="DELETE")

    def __repr__(self):
        return f"ApiToken(id={self.id}, owner={self.owner}, name={self.name})"


class ApiTokenCreated(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.token: str = raw_element.get("token")
        self.expiration_date: Optional[datetime] = iso8601_to_datetime(raw_element.get("expirationDate"))
        self.id: str = raw_element.get("id")



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import warnings
import functools
from datetime import datetime, timezone
from typing import Union, Optional
import unicodedata
import re


ISO_8601 = "%Y-%m-%dT%H:%M:%S.%fZ"


def slugify(value):
    value = str(value)
    value = unicodedata.normalize("NFKC", value)
    value = re.sub(r"[:.-/\s]+", "_", value)
    value = re.sub(r"[^\w\s-]", "", value)
    return value


def deprecated(reason=""):
    def decorator(func):
        @functools.wraps(func)
        def new_func(*args, **kwargs):
            warnings.warn(f"'{func.__name__}' is deprecated. {reason}", category=DeprecationWarning, stacklevel=2)
            return func(*args, **kwargs)

        return new_func

    return decorator


def timestamp_to_string(timestamp: Optional[Union[datetime, str]]) -> Optional[str]:
    if not isinstance(timestamp, datetime):
        return timestamp
    return timestamp.isoformat(timespec="milliseconds")


def iso8601_to_datetime(timestamp: Optional[str]) -> Optional[datetime]:
    if isinstance(timestamp, str):
        return datetime.strptime(timestamp, ISO_8601)
    return timestamp


def int64_to_datetime(timestamp: Optional[int]) -> Optional[datetime]:
    if timestamp is None or not timestamp:
        return None
    return datetime.fromtimestamp(timestamp / 1000, timezone.utc)


def datetime_to_int64(timestamp: Optional[datetime]) -> Optional[int]:
    if not isinstance(timestamp, datetime):
        return timestamp
    return int(timestamp.replace(tzinfo=timezone.utc).timestamp() * 1000)



from dynatrace import Dynatrace
from dynatrace.environment_v1.smartscape_hosts import Host, MonitoringMode, OSArchitecture
from dynatrace.pagination import HeaderPaginatedList
from dynatrace.environment_v1.smartscape_hosts import Host


def test_list(dt: Dynatrace):
    hosts = dt.smartscape_hosts.list(page_size=20)
    assert isinstance(hosts, HeaderPaginatedList)
    for host in hosts:
        assert isinstance(host, Host)
        assert host.entity_id == "HOST-7EC661999923A6B9"
        assert host.discovered_name == "TAG009444368559.clients.example.com"
        assert host.last_seen_timestamp == 1621519976487
        for tag in host.tags:
            assert tag.context == "CONTEXTLESS"
            assert tag.key == "APP1234567"
            break
        assert host.os_version == "Windows 10 Enterprise 20H2 2009, ver. 10.0.19042"
        assert host.monitoring_mode == MonitoringMode.FULL_STACK
        assert host.consumed_host_units == 2.0
        assert host.os_architecture == OSArchitecture.X_EIGHTY_SIX
        assert host.cpu_cores == 8
        break



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from enum import Enum
from requests import Response
from typing import Dict, Any, Optional

from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient
from dynatrace.environment_v2.schemas import ConfigurationMetadata
from dynatrace.configuration_v1.schemas import AutoUpdateSetting, UpdateWindowsConfig, EffectiveSetting, TechMonitoringList


class OneAgentOnAHostService:
    ENDPOINT = "/api/config/v1/hosts"

    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def get(self, host_id: str) -> "HostConfig":
        """Gets the full OneAgent configuration on the specified host

        :param host_id: The Dynatrace entity ID of the required host.

        :returns HostConfig: The full OneAgent configuration details on this host
        """
        response = self.__http_client.make_request(path=f"{self.ENDPOINT}/{host_id}").json()
        return HostConfig(raw_element=response, http_client=self.__http_client)

    def get_autoupdate(self, host_id: str) -> "HostAutoUpdateConfig":
        """Gets the configuration of OneAgent auto-update on the specified host

        :param host_id: The Dynatrace entity ID of the required host.

        :returns HostAutoUpdateConfig: The auto-update configuration details on this host
        """
        response = self.__http_client.make_request(path=f"{self.ENDPOINT}/{host_id}/autoupdate").json()
        return HostAutoUpdateConfig(raw_element=response, http_client=self.__http_client)

    def get_monitoring(self, host_id: str) -> "MonitoringConfig":
        """Gets the monitoring configuration of OneAgent on the specified host

        :param host_id: The Dynatrace entity ID of the required host.

        :returns MonitoringConfig: The monitoring configuration details on this host
        """
        response = self.__http_client.make_request(path=f"{self.ENDPOINT}/{host_id}/monitoring").json()
        return MonitoringConfig(raw_element=response, http_client=self.__http_client)

    def get_technologies(self, host_id: str) -> "TechMonitoringList":
        """Gets the configuration of monitored technologies on the specified host

        :param host_id: The Dynatrace entity ID of the required host.

        :returns TechMonitoringList: The monitored technologies configuration details on this host
        """
        response = self.__http_client.make_request(path=f"{self.ENDPOINT}/{host_id}/technologies").json()
        return TechMonitoringList(raw_element=response)

    def put_autoupdate(self, host_id: str, config: "HostAutoUpdateConfig") -> "Response":
        """Updates the configuration of OneAgent auto-update on the specified host.

        OneAgent is updated several minutes after the change of configuration.
        The process can take more time depending on number of OneAgents in the update queue.

        :param host_id: The Dynatrace entity ID of the required host.
        :param config: the updated auto-update configuration object

        :returns Response: HTTP Response to the request
        """
        return self.__http_client.make_request(path=f"{self.ENDPOINT}/{host_id}/autoupdate", method="PUT", params=config.to_json())

    def put_monitoring(self, host_id: str, config: "MonitoringConfig") -> "Response":
        """Updates the monitoring configuration of OneAgent on the specified host.

        The monitoring mode of OneAgent is updated several minutes after the change of configuration.

        :param host_id: The Dynatrace entity ID of the required host.
        :param config: the updated monitoring configuration object

        :returns Response: HTTP Response to the request
        """
        return self.__http_client.make_request(path=f"{self.ENDPOINT}/{host_id}/monitoring", method="PUT", params=config.to_json())

    def is_valid_autoupdate(self, host_id: str, config: "HostAutoUpdateConfig") -> bool:
        """Validates the payload for the put_autoupdate function

        :param host_id: The Dynatrace entity ID of the required host.
        :param config: the auto-update configuration object to validate

        :returns bool: True if valid, False otherwise
        """
        try:
            self.__http_client.make_request(path=f"{self.ENDPOINT}/{host_id}/autoupdate/validator", method="POST", params=config.to_json())
        except Exception as e:
            print(e.args)
            return False
        else:
            return True

    def is_valid_monitoring(self, host_id: str, config: "MonitoringConfig") -> bool:
        """Validates the payload for the put_monitoring function

        :param host_id: The Dynatrace entity ID of the required host.
        :param config: the monitoring configuration object to validate

        :returns bool: True if valid, False otherwise
        """
        try:
            self.__http_client.make_request(path=f"{self.ENDPOINT}/{host_id}/monitoring/validator", method="POST", params=config.to_json())
        except Exception as e:
            print(e.args)
            return False
        else:
            return True


class HostConfig(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.id: str = raw_element.get("id", "")
        self.monitoring_config: MonitoringConfig = MonitoringConfig(raw_element=raw_element.get("monitoringConfig"), http_client=self._http_client)
        self.auto_update_config: HostAutoUpdateConfig = HostAutoUpdateConfig(raw_element=raw_element.get("autoUpdateConfig"), http_client=self._http_client)
        self.tech_monitoring_config_list: TechMonitoringList = TechMonitoringList(raw_element=raw_element.get("techMonitoringConfigList"))


class MonitoringConfig(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.id: str = raw_element.get("id", "")
        self.monitoring_enabled: bool = raw_element.get("monitoringEnabled", False)
        self.monitoring_mode: MonitoringMode = MonitoringMode(raw_element.get("monitoringMode"))
        self.metadata: ConfigurationMetadata = ConfigurationMetadata(raw_element=raw_element.get("metadata"))

    def to_json(self) -> Dict[str, Any]:
        return {
            "monitoringEnabled": str(self.monitoring_enabled),
            "monitoringMode": str(self.monitoring_mode),
        }

    def put(self) -> "Response":
        return self._http_client.make_request(path=f"{OneAgentOnAHostService.ENDPOINT}/{self.id}/monitoring", method="PUT", params=self.to_json())


class HostAutoUpdateConfig(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.id: str = raw_element.get("id", "")
        self.setting: AutoUpdateSetting = AutoUpdateSetting(raw_element.get("setting"))
        self.version: Optional[str] = raw_element.get("version")
        self.update_windows: UpdateWindowsConfig = UpdateWindowsConfig(raw_element=raw_element.get("updateWindows"))
        self.effective_setting: Optional[EffectiveSetting] = None
        self.effective_version: Optional[str] = raw_element.get("effectiveVersion")
        self.metadata: ConfigurationMetadata = ConfigurationMetadata(raw_element=raw_element.get("metadata"))

        if raw_element.get("effectiveSetting"):
            self.effective_setting = EffectiveSetting(raw_element.get("effectiveSetting"))

    def to_json(self) -> Dict[str, Any]:
        return {
            "setting": str(self.setting),
            "version": self.version,
            "updateWindows": self.update_windows.to_json(),
            "effectiveVersion": self.effective_version,
        }

    def put(self) -> "Response":
        return self._http_client.make_request(path=f"{OneAgentOnAHostService.ENDPOINT}/{self.id}/autoupdate", method="PUT", params=self.to_json())


class MonitoringMode(Enum):
    CLOUD_INFRASTRUCTURE = "CLOUD_INFRASTRUCTURE"
    FULL_STACK = "FULL_STACK"

    def __str__(self):
        return self.value



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from enum import Enum
from typing import List, Dict, Any, Optional
from dynatrace.dynatrace_object import DynatraceObject

# Schemas that don't belong to a specific api tag


class ConfigurationMetadata(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.cluster_version: str = raw_element.get("clusterVersion")
        self.configuration_versions: List[int] = raw_element.get("configurationVersions")
        self.current_configuration_versions: List[str] = raw_element.get("currentConfigurationVersions")


class VersionCompareType(Enum):
    EQUAL = "EQUAL"
    GREATER = "GREATER"
    GREATER_EQUAL = "GREATER_EQUAL"
    LOWER = "LOWER"
    LOWER_EQUAL = "LOWER_EQUAL"

    def __repr__(self) -> str:
        return self.value

    def __str__(self) -> str:
        return self.value


class ManagementZone(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.name: str = raw_element["name"]
        self.id: str = raw_element["id"]

    def to_json(self) -> Dict[str, Any]:
        return {"name": self.name, "id": self.id}



import hashlib
import os
from pathlib import Path
from typing import Optional, Dict
from unittest import mock
import json

import pytest

from dynatrace import Dynatrace
from dynatrace.http_client import HttpClient
from dynatrace.utils import slugify

current_file_path = os.path.dirname(os.path.realpath(__file__))


class MockResponse:
    def __init__(self, json_data):
        self.json_data = json_data
        self.headers = {}

    def json(self):
        return self.json_data


def local_make_request(
    self, path: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, method="GET", data=None, query_params=None, **kwargs
) -> MockResponse:

    params = f"{params}" if params else ""
    if query_params:
        params += f"{query_params}"
    if params:
        encoded = f"{params}".encode()
        params = f"_{hashlib.sha256(encoded).hexdigest()}"[:16]

    path = slugify(path)
    file_name = f"{method}{path}{params}.json"
    file_path = Path(current_file_path, "mock_data", file_name)
    with open(file_path) as f:
        json_data = json.load(f)
        return MockResponse(json_data)


@pytest.fixture(autouse=True)
def dt():
    with mock.patch.object(HttpClient, "make_request", new=local_make_request):
        dt = Dynatrace("mock_tenant", "mock_token")
        yield dt



from setuptools import setup, find_packages

setup(
    name="dt",
    version="1.1.47",
    packages=find_packages(),
    install_requires=["requests>=2.22"],
    tests_require=["pytest", "mock", "tox"],
    python_requires=">=3.6",
    author="David Lopes",
    author_email="davidribeirolopes@gmail.com",
    description="Dynatrace API Python client",
    long_description="Dynatrace API Python client",
    url="https://github.com/dlopes7/dynatrace-rest-python",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved",
        "License :: OSI Approved :: Apache Software License",  # 2.0
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows",
        "Topic :: Software Development",
    ],
    project_urls={"Issue Tracker": "https://github.com/dlopes7/dynatrace-rest-python/issues"},
)



from dynatrace import Dynatrace
from datetime import datetime


def test_list(dt: Dynatrace):
    update_jobs = list(dt.activegates_autoupdate_jobs.list())
    assert isinstance(update_jobs, list)

    first_ag = update_jobs[0]
    assert first_ag.activegate_id == "-1556499193"
    assert len(first_ag.update_jobs) == 3

    first_job = first_ag.update_jobs[0]
    assert first_job.job_id == "-4754066816153208332"
    assert first_job.job_state == "SUCCEED"
    assert first_job.update_method == "MANUAL_INSTALLATION"
    assert first_job.update_type == "ACTIVE_GATE"
    assert not first_job.cancelable
    assert first_job.start_version == "1.217.89.20210506-182520"
    assert first_job.target_version == "1.217.96.20210507-181038"
    assert first_job.timestamp == datetime.utcfromtimestamp(1620419177047 / 1000)
    assert first_job.ag_type == "ENVIRONMENT"
    assert first_job.environments == ["eaa50379"]
    assert first_job.error is None
    assert first_job.duration is None


def test_get(dt: Dynatrace):
    update_jobs_list = dt.activegates_autoupdate_jobs.get("-1556499193")

    assert update_jobs_list.activegate_id == "-1556499193"
    assert isinstance(update_jobs_list.update_jobs, list)

    first_job = update_jobs_list.update_jobs[0]
    assert first_job.job_id == "-4754066816153208332"
    assert first_job.job_state == "SUCCEED"
    assert first_job.update_method == "MANUAL_INSTALLATION"
    assert first_job.update_type == "ACTIVE_GATE"
    assert not first_job.cancelable
    assert first_job.start_version == "1.217.89.20210506-182520"
    assert first_job.target_version == "1.217.96.20210507-181038"
    assert first_job.timestamp == datetime.utcfromtimestamp(1620419177047 / 1000)
    assert first_job.ag_type == "ENVIRONMENT"
    assert first_job.environments == ["eaa50379"]
    assert first_job.error is None
    assert first_job.duration is None



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from datetime import datetime
from enum import Enum
from pathlib import Path
from typing import List, Dict, Any, Optional, Union
from dynatrace import http_client

from requests import Response

from dynatrace.environment_v2.schemas import ConfigurationMetadata
from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.environment_v2.monitored_entities import EntityShortRepresentation
from dynatrace.http_client import HttpClient
from dynatrace.pagination import PaginatedList


class AutoTagService:
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(self, page_size: int = 200) -> PaginatedList["AutoTagShortRepresentation"]:
        """
        List all auto tag rules.

        :param page_size: The number of results per result page. Must be between 1 and 500
            Default value : 200
        """
        params = {"pageSize": page_size}
        return PaginatedList(AutoTagShortRepresentation, self.__http_client, f"/api/config/v1/autoTags", params, list_item="values")


class PropagationType(Enum):
    SERVICE_TO_PROCESS_GROUP_LIKE = "SERVICE_TO_PROCESS_GROUP_LIKE"
    SERVICE_TO_HOST_LIKE = "SERVICE_TO_HOST_LIKE"
    PROCESS_GROUP_TO_HOST = "PROCESS_GROUP_TO_HOST"
    PROCESS_GROUP_TO_SERVICE = "PROCESS_GROUP_TO_SERVICE"
    HOST_TO_PROCESS_GROUP_INSTANCE = "HOST_TO_PROCESS_GROUP_INSTANCE"
    AZURE_TO_PG = "AZURE_TO_PG"
    AZURE_TO_SERVICE = "AZURE_TO_SERVICE"
    NONE = None


class ConditionKeyAttribute(Enum):
    APPMON_SERVER_NAME = "APPMON_SERVER_NAME"
    APPMON_SYSTEM_PROFILE_NAME = "APPMON_SYSTEM_PROFILE_NAME"
    AWS_ACCOUNT_ID = "AWS_ACCOUNT_ID"
    AWS_ACCOUNT_NAME = "AWS_ACCOUNT_NAME"
    AWS_APPLICATION_LOAD_BALANCER_NAME = "AWS_APPLICATION_LOAD_BALANCER_NAME"
    AWS_APPLICATION_LOAD_BALANCER_TAGS = "AWS_APPLICATION_LOAD_BALANCER_TAGS"
    AWS_AUTO_SCALING_GROUP_NAME = "AWS_AUTO_SCALING_GROUP_NAME"
    AWS_AUTO_SCALING_GROUP_TAGS = "AWS_AUTO_SCALING_GROUP_TAGS"
    AWS_AVAILABILITY_ZONE_NAME = "AWS_AVAILABILITY_ZONE_NAME"
    AWS_CLASSIC_LOAD_BALANCER_FRONTEND_PORTS = "AWS_CLASSIC_LOAD_BALANCER_FRONTEND_PORTS"
    AWS_CLASSIC_LOAD_BALANCER_NAME = "AWS_CLASSIC_LOAD_BALANCER_NAME"
    AWS_CLASSIC_LOAD_BALANCER_TAGS = "AWS_CLASSIC_LOAD_BALANCER_TAGS"
    AWS_NETWORK_LOAD_BALANCER_NAME = "AWS_NETWORK_LOAD_BALANCER_NAME"
    AWS_NETWORK_LOAD_BALANCER_TAGS = "AWS_NETWORK_LOAD_BALANCER_TAGS"
    AWS_RELATIONAL_DATABASE_SERVICE_DB_NAME = "AWS_RELATIONAL_DATABASE_SERVICE_DB_NAME"
    AWS_RELATIONAL_DATABASE_SERVICE_ENDPOINT = "AWS_RELATIONAL_DATABASE_SERVICE_ENDPOINT"
    AWS_RELATIONAL_DATABASE_SERVICE_ENGINE = "AWS_RELATIONAL_DATABASE_SERVICE_ENGINE"
    AWS_RELATIONAL_DATABASE_SERVICE_INSTANCE_CLASS = "AWS_RELATIONAL_DATABASE_SERVICE_INSTANCE_CLASS"
    AWS_RELATIONAL_DATABASE_SERVICE_NAME = "AWS_RELATIONAL_DATABASE_SERVICE_NAME"
    AWS_RELATIONAL_DATABASE_SERVICE_PORT = "AWS_RELATIONAL_DATABASE_SERVICE_PORT"
    AWS_RELATIONAL_DATABASE_SERVICE_TAGS = "AWS_RELATIONAL_DATABASE_SERVICE_TAGS"
    AZURE_ENTITY_NAME = "AZURE_ENTITY_NAME"
    AZURE_ENTITY_TAGS = "AZURE_ENTITY_TAGS"
    AZURE_MGMT_GROUP_NAME = "AZURE_MGMT_GROUP_NAME"
    AZURE_MGMT_GROUP_UUID = "AZURE_MGMT_GROUP_UUID"
    AZURE_REGION_NAME = "AZURE_REGION_NAME"
    AZURE_SCALE_SET_NAME = "AZURE_SCALE_SET_NAME"
    AZURE_SUBSCRIPTION_NAME = "AZURE_SUBSCRIPTION_NAME"
    AZURE_SUBSCRIPTION_UUID = "AZURE_SUBSCRIPTION_UUID"
    AZURE_TENANT_NAME = "AZURE_TENANT_NAME"
    AZURE_TENANT_UUID = "AZURE_TENANT_UUID"
    AZURE_VM_NAME = "AZURE_VM_NAME"
    BROWSER_MONITOR_NAME = "BROWSER_MONITOR_NAME"
    BROWSER_MONITOR_TAGS = "BROWSER_MONITOR_TAGS"
    CLOUD_APPLICATION_LABELS = "CLOUD_APPLICATION_LABELS"
    CLOUD_APPLICATION_NAME = "CLOUD_APPLICATION_NAME"
    CLOUD_APPLICATION_NAMESPACE_LABELS = "CLOUD_APPLICATION_NAMESPACE_LABELS"
    CLOUD_APPLICATION_NAMESPACE_NAME = "CLOUD_APPLICATION_NAMESPACE_NAME"
    CLOUD_FOUNDRY_FOUNDATION_NAME = "CLOUD_FOUNDRY_FOUNDATION_NAME"
    CLOUD_FOUNDRY_ORG_NAME = "CLOUD_FOUNDRY_ORG_NAME"
    CUSTOM_APPLICATION_NAME = "CUSTOM_APPLICATION_NAME"
    CUSTOM_APPLICATION_PLATFORM = "CUSTOM_APPLICATION_PLATFORM"
    CUSTOM_APPLICATION_TAGS = "CUSTOM_APPLICATION_TAGS"
    CUSTOM_APPLICATION_TYPE = "CUSTOM_APPLICATION_TYPE"
    CUSTOM_DEVICE_DETECTED_NAME = "CUSTOM_DEVICE_DETECTED_NAME"
    CUSTOM_DEVICE_DNS_ADDRESS = "CUSTOM_DEVICE_DNS_ADDRESS"
    CUSTOM_DEVICE_GROUP_NAME = "CUSTOM_DEVICE_GROUP_NAME"
    CUSTOM_DEVICE_GROUP_TAGS = "CUSTOM_DEVICE_GROUP_TAGS"
    CUSTOM_DEVICE_IP_ADDRESS = "CUSTOM_DEVICE_IP_ADDRESS"
    CUSTOM_DEVICE_METADATA = "CUSTOM_DEVICE_METADATA"
    CUSTOM_DEVICE_NAME = "CUSTOM_DEVICE_NAME"
    CUSTOM_DEVICE_PORT = "CUSTOM_DEVICE_PORT"
    CUSTOM_DEVICE_TAGS = "CUSTOM_DEVICE_TAGS"
    CUSTOM_DEVICE_TECHNOLOGY = "CUSTOM_DEVICE_TECHNOLOGY"
    DATA_CENTER_SERVICE_DECODER_TYPE = "DATA_CENTER_SERVICE_DECODER_TYPE"
    DATA_CENTER_SERVICE_IP_ADDRESS = "DATA_CENTER_SERVICE_IP_ADDRESS"
    DATA_CENTER_SERVICE_METADATA = "DATA_CENTER_SERVICE_METADATA"
    DATA_CENTER_SERVICE_NAME = "DATA_CENTER_SERVICE_NAME"
    DATA_CENTER_SERVICE_PORT = "DATA_CENTER_SERVICE_PORT"
    DATA_CENTER_SERVICE_TAGS = "DATA_CENTER_SERVICE_TAGS"
    DOCKER_CONTAINER_NAME = "DOCKER_CONTAINER_NAME"
    DOCKER_FULL_IMAGE_NAME = "DOCKER_FULL_IMAGE_NAME"
    DOCKER_IMAGE_VERSION = "DOCKER_IMAGE_VERSION"
    DOCKER_STRIPPED_IMAGE_NAME = "DOCKER_STRIPPED_IMAGE_NAME"
    EC2_INSTANCE_AMI_ID = "EC2_INSTANCE_AMI_ID"
    EC2_INSTANCE_AWS_INSTANCE_TYPE = "EC2_INSTANCE_AWS_INSTANCE_TYPE"
    EC2_INSTANCE_AWS_SECURITY_GROUP = "EC2_INSTANCE_AWS_SECURITY_GROUP"
    EC2_INSTANCE_BEANSTALK_ENV_NAME = "EC2_INSTANCE_BEANSTALK_ENV_NAME"
    EC2_INSTANCE_ID = "EC2_INSTANCE_ID"
    EC2_INSTANCE_NAME = "EC2_INSTANCE_NAME"
    EC2_INSTANCE_PRIVATE_HOST_NAME = "EC2_INSTANCE_PRIVATE_HOST_NAME"
    EC2_INSTANCE_PUBLIC_HOST_NAME = "EC2_INSTANCE_PUBLIC_HOST_NAME"
    EC2_INSTANCE_TAGS = "EC2_INSTANCE_TAGS"
    ENTERPRISE_APPLICATION_DECODER_TYPE = "ENTERPRISE_APPLICATION_DECODER_TYPE"
    ENTERPRISE_APPLICATION_IP_ADDRESS = "ENTERPRISE_APPLICATION_IP_ADDRESS"
    ENTERPRISE_APPLICATION_METADATA = "ENTERPRISE_APPLICATION_METADATA"
    ENTERPRISE_APPLICATION_NAME = "ENTERPRISE_APPLICATION_NAME"
    ENTERPRISE_APPLICATION_PORT = "ENTERPRISE_APPLICATION_PORT"
    ENTERPRISE_APPLICATION_TAGS = "ENTERPRISE_APPLICATION_TAGS"
    ESXI_HOST_CLUSTER_NAME = "ESXI_HOST_CLUSTER_NAME"
    ESXI_HOST_HARDWARE_MODEL = "ESXI_HOST_HARDWARE_MODEL"
    ESXI_HOST_HARDWARE_VENDOR = "ESXI_HOST_HARDWARE_VENDOR"
    ESXI_HOST_NAME = "ESXI_HOST_NAME"
    ESXI_HOST_PRODUCT_NAME = "ESXI_HOST_PRODUCT_NAME"
    ESXI_HOST_PRODUCT_VERSION = "ESXI_HOST_PRODUCT_VERSION"
    ESXI_HOST_TAGS = "ESXI_HOST_TAGS"
    EXTERNAL_MONITOR_ENGINE_DESCRIPTION = "EXTERNAL_MONITOR_ENGINE_DESCRIPTION"
    EXTERNAL_MONITOR_ENGINE_NAME = "EXTERNAL_MONITOR_ENGINE_NAME"
    EXTERNAL_MONITOR_ENGINE_TYPE = "EXTERNAL_MONITOR_ENGINE_TYPE"
    EXTERNAL_MONITOR_NAME = "EXTERNAL_MONITOR_NAME"
    EXTERNAL_MONITOR_TAGS = "EXTERNAL_MONITOR_TAGS"
    GEOLOCATION_SITE_NAME = "GEOLOCATION_SITE_NAME"
    GOOGLE_CLOUD_PLATFORM_ZONE_NAME = "GOOGLE_CLOUD_PLATFORM_ZONE_NAME"
    GOOGLE_COMPUTE_INSTANCE_ID = "GOOGLE_COMPUTE_INSTANCE_ID"
    GOOGLE_COMPUTE_INSTANCE_MACHINE_TYPE = "GOOGLE_COMPUTE_INSTANCE_MACHINE_TYPE"
    GOOGLE_COMPUTE_INSTANCE_NAME = "GOOGLE_COMPUTE_INSTANCE_NAME"
    GOOGLE_COMPUTE_INSTANCE_PROJECT = "GOOGLE_COMPUTE_INSTANCE_PROJECT"
    GOOGLE_COMPUTE_INSTANCE_PROJECT_ID = "GOOGLE_COMPUTE_INSTANCE_PROJECT_ID"
    GOOGLE_COMPUTE_INSTANCE_PUBLIC_IP_ADDRESSES = "GOOGLE_COMPUTE_INSTANCE_PUBLIC_IP_ADDRESSES"
    HOST_AIX_LOGICAL_CPU_COUNT = "HOST_AIX_LOGICAL_CPU_COUNT"
    HOST_AIX_SIMULTANEOUS_THREADS = "HOST_AIX_SIMULTANEOUS_THREADS"
    HOST_AIX_VIRTUAL_CPU_COUNT = "HOST_AIX_VIRTUAL_CPU_COUNT"
    HOST_ARCHITECTURE = "HOST_ARCHITECTURE"
    HOST_AWS_NAME_TAG = "HOST_AWS_NAME_TAG"
    HOST_AZURE_COMPUTE_MODE = "HOST_AZURE_COMPUTE_MODE"
    HOST_AZURE_SKU = "HOST_AZURE_SKU"
    HOST_AZURE_WEB_APPLICATION_HOST_NAMES = "HOST_AZURE_WEB_APPLICATION_HOST_NAMES"
    HOST_AZURE_WEB_APPLICATION_SITE_NAMES = "HOST_AZURE_WEB_APPLICATION_SITE_NAMES"
    HOST_BITNESS = "HOST_BITNESS"
    HOST_BOSH_AVAILABILITY_ZONE = "HOST_BOSH_AVAILABILITY_ZONE"
    HOST_BOSH_DEPLOYMENT_ID = "HOST_BOSH_DEPLOYMENT_ID"
    HOST_BOSH_INSTANCE_ID = "HOST_BOSH_INSTANCE_ID"
    HOST_BOSH_INSTANCE_NAME = "HOST_BOSH_INSTANCE_NAME"
    HOST_BOSH_NAME = "HOST_BOSH_NAME"
    HOST_BOSH_STEMCELL_VERSION = "HOST_BOSH_STEMCELL_VERSION"
    HOST_CLOUD_TYPE = "HOST_CLOUD_TYPE"
    HOST_CPU_CORES = "HOST_CPU_CORES"
    HOST_CUSTOM_METADATA = "HOST_CUSTOM_METADATA"
    HOST_DETECTED_NAME = "HOST_DETECTED_NAME"
    HOST_GROUP_ID = "HOST_GROUP_ID"
    HOST_GROUP_NAME = "HOST_GROUP_NAME"
    HOST_HYPERVISOR_TYPE = "HOST_HYPERVISOR_TYPE"
    HOST_IP_ADDRESS = "HOST_IP_ADDRESS"
    HOST_KUBERNETES_LABELS = "HOST_KUBERNETES_LABELS"
    HOST_LOGICAL_CPU_CORES = "HOST_LOGICAL_CPU_CORES"
    HOST_NAME = "HOST_NAME"
    HOST_ONEAGENT_CUSTOM_HOST_NAME = "HOST_ONEAGENT_CUSTOM_HOST_NAME"
    HOST_OS_TYPE = "HOST_OS_TYPE"
    HOST_OS_VERSION = "HOST_OS_VERSION"
    HOST_PAAS_MEMORY_LIMIT = "HOST_PAAS_MEMORY_LIMIT"
    HOST_PAAS_TYPE = "HOST_PAAS_TYPE"
    HOST_TAGS = "HOST_TAGS"
    HOST_TECHNOLOGY = "HOST_TECHNOLOGY"
    HTTP_MONITOR_NAME = "HTTP_MONITOR_NAME"
    HTTP_MONITOR_TAGS = "HTTP_MONITOR_TAGS"
    KUBERNETES_CLUSTER_NAME = "KUBERNETES_CLUSTER_NAME"
    KUBERNETES_NODE_NAME = "KUBERNETES_NODE_NAME"
    MOBILE_APPLICATION_NAME = "MOBILE_APPLICATION_NAME"
    MOBILE_APPLICATION_PLATFORM = "MOBILE_APPLICATION_PLATFORM"
    MOBILE_APPLICATION_TAGS = "MOBILE_APPLICATION_TAGS"
    NAME_OF_COMPUTE_NODE = "NAME_OF_COMPUTE_NODE"
    OPENSTACK_ACCOUNT_NAME = "OPENSTACK_ACCOUNT_NAME"
    OPENSTACK_ACCOUNT_PROJECT_NAME = "OPENSTACK_ACCOUNT_PROJECT_NAME"
    OPENSTACK_AVAILABILITY_ZONE_NAME = "OPENSTACK_AVAILABILITY_ZONE_NAME"
    OPENSTACK_PROJECT_NAME = "OPENSTACK_PROJECT_NAME"
    OPENSTACK_REGION_NAME = "OPENSTACK_REGION_NAME"
    OPENSTACK_VM_INSTANCE_TYPE = "OPENSTACK_VM_INSTANCE_TYPE"
    OPENSTACK_VM_NAME = "OPENSTACK_VM_NAME"
    OPENSTACK_VM_SECURITY_GROUP = "OPENSTACK_VM_SECURITY_GROUP"
    PROCESS_GROUP_AZURE_HOST_NAME = "PROCESS_GROUP_AZURE_HOST_NAME"
    PROCESS_GROUP_AZURE_SITE_NAME = "PROCESS_GROUP_AZURE_SITE_NAME"
    PROCESS_GROUP_CUSTOM_METADATA = "PROCESS_GROUP_CUSTOM_METADATA"
    PROCESS_GROUP_DETECTED_NAME = "PROCESS_GROUP_DETECTED_NAME"
    PROCESS_GROUP_ID = "PROCESS_GROUP_ID"
    PROCESS_GROUP_LISTEN_PORT = "PROCESS_GROUP_LISTEN_PORT"
    PROCESS_GROUP_NAME = "PROCESS_GROUP_NAME"
    PROCESS_GROUP_PREDEFINED_METADATA = "PROCESS_GROUP_PREDEFINED_METADATA"
    PROCESS_GROUP_TAGS = "PROCESS_GROUP_TAGS"
    PROCESS_GROUP_TECHNOLOGY = "PROCESS_GROUP_TECHNOLOGY"
    PROCESS_GROUP_TECHNOLOGY_EDITION = "PROCESS_GROUP_TECHNOLOGY_EDITION"
    PROCESS_GROUP_TECHNOLOGY_VERSION = "PROCESS_GROUP_TECHNOLOGY_VERSION"
    SERVICE_AKKA_ACTOR_SYSTEM = "SERVICE_AKKA_ACTOR_SYSTEM"
    SERVICE_CTG_SERVICE_NAME = "SERVICE_CTG_SERVICE_NAME"
    SERVICE_DATABASE_HOST_NAME = "SERVICE_DATABASE_HOST_NAME"
    SERVICE_DATABASE_NAME = "SERVICE_DATABASE_NAME"
    SERVICE_DATABASE_TOPOLOGY = "SERVICE_DATABASE_TOPOLOGY"
    SERVICE_DATABASE_VENDOR = "SERVICE_DATABASE_VENDOR"
    SERVICE_DETECTED_NAME = "SERVICE_DETECTED_NAME"
    SERVICE_ESB_APPLICATION_NAME = "SERVICE_ESB_APPLICATION_NAME"
    SERVICE_IBM_CTG_GATEWAY_URL = "SERVICE_IBM_CTG_GATEWAY_URL"
    SERVICE_IIB_APPLICATION_NAME = "SERVICE_IIB_APPLICATION_NAME"
    SERVICE_MESSAGING_LISTENER_CLASS_NAME = "SERVICE_MESSAGING_LISTENER_CLASS_NAME"
    SERVICE_NAME = "SERVICE_NAME"
    SERVICE_PORT = "SERVICE_PORT"
    SERVICE_PUBLIC_DOMAIN_NAME = "SERVICE_PUBLIC_DOMAIN_NAME"
    SERVICE_REMOTE_ENDPOINT = "SERVICE_REMOTE_ENDPOINT"
    SERVICE_REMOTE_SERVICE_NAME = "SERVICE_REMOTE_SERVICE_NAME"
    SERVICE_TAGS = "SERVICE_TAGS"
    SERVICE_TECHNOLOGY = "SERVICE_TECHNOLOGY"
    SERVICE_TECHNOLOGY_EDITION = "SERVICE_TECHNOLOGY_EDITION"
    SERVICE_TECHNOLOGY_VERSION = "SERVICE_TECHNOLOGY_VERSION"
    SERVICE_TOPOLOGY = "SERVICE_TOPOLOGY"
    SERVICE_TYPE = "SERVICE_TYPE"
    SERVICE_WEB_APPLICATION_ID = "SERVICE_WEB_APPLICATION_ID"
    SERVICE_WEB_CONTEXT_ROOT = "SERVICE_WEB_CONTEXT_ROOT"
    SERVICE_WEB_SERVER_ENDPOINT = "SERVICE_WEB_SERVER_ENDPOINT"
    SERVICE_WEB_SERVER_NAME = "SERVICE_WEB_SERVER_NAME"
    SERVICE_WEB_SERVICE_NAME = "SERVICE_WEB_SERVICE_NAME"
    SERVICE_WEB_SERVICE_NAMESPACE = "SERVICE_WEB_SERVICE_NAMESPACE"
    VMWARE_DATACENTER_NAME = "VMWARE_DATACENTER_NAME"
    VMWARE_VM_NAME = "VMWARE_VM_NAME"
    WEB_APPLICATION_NAME = "WEB_APPLICATION_NAME"
    WEB_APPLICATION_NAME_PATTERN = "WEB_APPLICATION_NAME_PATTERN"
    WEB_APPLICATION_TAGS = "WEB_APPLICATION_TAGS"
    WEB_APPLICATION_TYPE = "WEB_APPLICATION_TYPE"
    NONE = None


class ConditionKeyType(Enum):
    PROCESS_CUSTOM_METADATA_KEY = "PROCESS_CUSTOM_METADATA_KEY"
    HOST_CUSTOM_METADATA_KEY = "HOST_CUSTOM_METADATA_KEY"
    PROCESS_PREDEFINED_METADATA_KEY = "PROCESS_PREDEFINED_METADATA_KEY"
    STRING = "STRING"
    STATIC = "STATIC"
    NONE = None


class ComparisonBasicType(Enum):
    APPLICATION_TYPE = "APPLICATION_TYPE"
    AZURE_COMPUTE_MODE = "AZURE_COMPUTE_MODE"
    AZURE_SKU = "AZURE_SKU"
    BITNESS = "BITNESS"
    CLOUD_TYPE = "CLOUD_TYPE"
    CUSTOM_APPLICATION_TYPE = "CUSTOM_APPLICATION_TYPE"
    DATABASE_TOPOLOGY = "DATABASE_TOPOLOGY"
    DCRUM_DECODER_TYPE = "DCRUM_DECODER_TYPE"
    ENTITY_ID = "ENTITY_ID"
    HYPERVISOR_TYPE = "HYPERVISOR_TYPE"
    INDEXED_NAME = "INDEXED_NAME"
    INDEXED_STRING = "INDEXED_STRING"
    INDEXED_TAG = "INDEXED_TAG"
    INTEGER = "INTEGER"
    IP_ADDRESS = "IP_ADDRESS"
    MOBILE_PLATFORM = "MOBILE_PLATFORM"
    OS_ARCHITECTURE = "OS_ARCHITECTURE"
    OS_TYPE = "OS_TYPE"
    PAAS_TYPE = "PAAS_TYPE"
    SERVICE_TOPOLOGY = "SERVICE_TOPOLOGY"
    SERVICE_TYPE = "SERVICE_TYPE"
    SIMPLE_HOST_TECH = "SIMPLE_HOST_TECH"
    SIMPLE_TECH = "SIMPLE_TECH"
    STRING = "STRING"
    SYNTHETIC_ENGINE_TYPE = "SYNTHETIC_ENGINE_TYPE"
    TAG = "TAG"
    NONE = None


class AutoTagRuleType(Enum):
    APPLICATION = "APPLICATION"
    AWS_APPLICATION_LOAD_BALANCER = "AWS_APPLICATION_LOAD_BALANCER"
    AWS_CLASSIC_LOAD_BALANCER = "AWS_CLASSIC_LOAD_BALANCER"
    AWS_NETWORK_LOAD_BALANCER = "AWS_NETWORK_LOAD_BALANCER"
    AWS_RELATIONAL_DATABASE_SERVICE = "AWS_RELATIONAL_DATABASE_SERVICE"
    AZURE = "AZURE"
    CUSTOM_APPLICATION = "CUSTOM_APPLICATION"
    CUSTOM_DEVICE = "CUSTOM_DEVICE"
    DCRUM_APPLICATION = "DCRUM_APPLICATION"
    ESXI_HOST = "ESXI_HOST"
    EXTERNAL_SYNTHETIC_TEST = "EXTERNAL_SYNTHETIC_TEST"
    HOST = "HOST"
    HTTP_CHECK = "HTTP_CHECK"
    MOBILE_APPLICATION = "MOBILE_APPLICATION"
    PROCESS_GROUP = "PROCESS_GROUP"
    SERVICE = "SERVICE"
    SYNTHETIC_TEST = "SYNTHETIC_TEST"
    NONE = None


class ComparisonBasic(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.operator: str = raw_element.get("operator")
        self.value: dict = raw_element.get("value")
        self.negate: bool = raw_element.get("negate")
        self.type: ComparisonBasicType = ComparisonBasicType(raw_element.get("type"))
        self.case_sensitive: bool = raw_element.get("caseSensitive")


class ConditionKey(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.attribute: ConditionKeyAttribute = ConditionKeyAttribute(raw_element.get("attribute"))
        self.type: ConditionKeyType = ConditionKeyType(raw_element.get("type"))


class EntityRuleEngineCondition(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.key: ConditionKey = ConditionKey(raw_element=raw_element.get("key"))
        self.comparison_info: ComparisonBasic = ComparisonBasic(raw_element=raw_element.get("comparisonInfo"))


class EntitySelectorBasedAutoTagRule(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.enabled: bool = raw_element.get("enabled")
        self.entity_selector: str = raw_element.get("entitySelector")
        self.value_format: str = raw_element.get("valueFormat")


class AutoTagRule(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.type: AutoTagRuleType = AutoTagRuleType(raw_element.get("type"))
        self.enabled: bool = raw_element.get("enabled")
        self.value_format: str = raw_element.get("valueFormat")
        self.propagation_types: List[PropagationType] = [PropagationType(prop_type) for prop_type in (raw_element.get("propagationTypes") or [])]
        self.conditions: List[EntityRuleEngineCondition] = [
            EntityRuleEngineCondition(raw_element=condition) for condition in (raw_element.get("conditions") or [])
        ]


class AutoTag(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.metadata: ConfigurationMetadata = ConfigurationMetadata(self._http_client, None, raw_element.get("metadata"))
        self.id: str = raw_element.get("id")
        self.name: str = raw_element.get("name")
        self.description: str = raw_element.get("description")
        self.rules: List[AutoTagRule] = [AutoTagRule(raw_element=rule) for rule in raw_element.get("rules")]
        self.entity_selector_based_rules: List[EntitySelectorBasedAutoTagRule] = [
            EntitySelectorBasedAutoTagRule(raw_element=rule) for rule in (raw_element.get("entitySelectorBasedRules") or [])
        ]


class AutoTagShortRepresentation(EntityShortRepresentation):
    def get_full_configuration(self):
        """
        Get the full configuration for this auto tag rule short representation.
        """
        response = self._http_client.make_request(f"/api/config/v1/autoTags/{self.id}").json()
        return AutoTag(http_client=self._http_client, raw_element=response)



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from dynatrace.dynatrace_object import DynatraceObject
from pathlib import Path
from typing import List, Dict, Any, Optional, Union

from dynatrace.http_client import HttpClient
from dynatrace.pagination import PaginatedList


class ExtensionsServiceV2:
    ENDPOINT = "/api/v2/extensions"
    SCHEMA_ENDPOINT = "/api/v2/extensions/schemas"

    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(self, name: Optional[str] = None) -> PaginatedList["MinimalExtension"]:
        """Lists all the extensions 2.0 available in your environment"""
        params = {"name": name}
        return PaginatedList(MinimalExtension, self.__http_client, target_url=self.ENDPOINT, list_item="extensions", target_params=params)

    def list_versions(self, extension_name: str) -> PaginatedList["MinimalExtension"]:
        """Lists all the extensions 2.0 by specified name in your environment

        :param extension_name: the name of the extension 2.0
        """
        return PaginatedList(MinimalExtension, self.__http_client, target_url=f"{self.ENDPOINT}/{extension_name}", list_item="extensions")

    def list_environment_config_events(self, extension_name: str) -> PaginatedList["ExtensionEventDTO"]:
        """List of the latest extension environment configuration events

        :param extension_name: the name of the extension 2.0

        :return: a list of ExtensionEventDTO object
        """
        return PaginatedList(
            ExtensionEventDTO, self.__http_client, target_url=f"{self.ENDPOINT}/{extension_name}/environmentConfiguration/events", list_item="extensionEvents"
        )

    def list_monitoring_config_events(self, extension_name: str, config_id: str) -> PaginatedList["ExtensionEventDTO"]:
        """Gets the list of the events linked to specific monitoring configuration

        :param extension_name: the name of the extension 2.0
        :param config_id: The ID of the requested monitoring configuration.

        :return: a list of ExtensionEventDTO object
        """
        return PaginatedList(
            ExtensionEventDTO,
            self.__http_client,
            target_url=f"{self.ENDPOINT}/{extension_name}/monitoringConfigurations/{config_id}/events",
            list_item="extensionEvents",
        )

    def get(self, extension_name: str, extension_version: str) -> "Extension":
        """Gets details of specified version of the extension 2.0

        :param extension_name: the name of the requested extension 2.0
        :param extension_version: the version of the requested extension 2.0

        :return: a Extension class object
        """
        response = self.__http_client.make_request(f"{self.ENDPOINT}/{extension_name}/{extension_version}").json()
        return Extension(raw_element=response)

    def post(self, zip_file_path: Union[str, Path], validate_only: Optional[bool] = False):
        """Post the specified version of the extension 2.0

        :param zip_file_path: path to zipped extension 2.0
        :param validate_only: optionally run validation but do not persist the extension even if validation was successful

        :return: newly created Extension class object
        """
        params = {"validateOnly": validate_only}
        file = Path(zip_file_path)
        with open(file, "rb") as f:
            response = self.__http_client.make_request(f"{self.ENDPOINT}", params=params, method="POST", files={"file": f}).json()
            return Extension(raw_element=response)

    def delete(self, extension_name: str, extension_version: str):
        """Deletes the specified version of the extension 2.0

        :param extension_name: the name of the requested extension 2.0
        :param extension_version: the version of the requested extension 2.0

        :return: HTTP response
        """
        return self.__http_client.make_request(f"{self.ENDPOINT}/{extension_name}/{extension_version}", method="DELETE")

    def get_environment_config(self, extension_name: str) -> "ExtensionEnvironmentConfigurationVersion":
        """Gets the active environment configuration version of the specified extension 2.0

        :param extension_name: the name of the requested extension 2.0

        :return: ExtensionEnvironmentConfigurationVersion object
        """
        response = self.__http_client.make_request(f"{self.ENDPOINT}/{extension_name}/environmentConfiguration").json()
        return ExtensionEnvironmentConfigurationVersion(raw_element=response)

    def put_environment_config(self, extension_name: str, extension_version: str):
        """Updates an existing active environment configuration version of the extension 2.0

        :param extension_name: the name of the requested extension 2.0
        :param extension_version: the version of the requested extension 2.0

        :return: HTTP response
        """
        params = {"version": extension_version}
        return self.__http_client.make_request(f"{self.ENDPOINT}/{extension_name}/environmentConfiguration", method="PUT", params=params)

    def delete_environment_config(self, extension_name: str):
        """Deactivates the environment configuration of the specified extension 2.0

        :param extension_name: the name of the requested extension 2.0 to deactivate

        :return: HTTP response
        """
        return self.__http_client.make_request(f"{self.ENDPOINT}/{extension_name}/environmentConfiguration", method="DELETE")

    def list_schemas_versions(self) -> List[str]:
        response = self.__http_client.make_request(f"{self.SCHEMA_ENDPOINT}").json()
        return response.get("versions", [])

    def list_schemas(self, schema_version: str) -> "SchemaFiles":
        response = self.__http_client.make_request(f"{self.SCHEMA_ENDPOINT}/{schema_version}")
        return SchemaFiles(raw_element=response.json())

    def get_schema_file(self, schema_version: str, file_name: str) -> Dict[str, Any]:
        return self.__http_client.make_request(f"{self.SCHEMA_ENDPOINT}/{schema_version}/{file_name}").json()

    def post_monitoring_configurations(self, extension_name: str, configurations: List["MonitoringConfigurationDto"]) -> List:
        params = [c.to_json() for c in configurations]
        response = self.__http_client.make_request(f"{self.ENDPOINT}/{extension_name}/monitoringConfigurations", params=params, method="POST")
        return response.json()


class SchemaFiles(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.files: List[str] = raw_element.get("files", [])


class Extension(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.version: str = raw_element.get("version")
        self.extension_name: str = raw_element.get("extensionName")
        self.min_dynatrace_version: str = raw_element.get("minDynatraceVersion")
        self.file_hash: str = raw_element.get("fileHash")
        self.author: AuthorDTO = AuthorDTO(raw_element=raw_element.get("author"))
        self.data_sources: List[str] = raw_element.get("dataSources")
        self.variables: List[str] = raw_element.get("variables")
        self.feature_sets: List[str] = raw_element.get("featureSets")


class AuthorDTO(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.name: str = raw_element.get("name")


class ExtensionEventDTO(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.timestamp: str = raw_element.get("timestamp")
        self.severity: str = raw_element.get("severity")
        self.message: str = raw_element.get("message")


class ExtensionEnvironmentConfigurationVersion(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.version: str = raw_element.get("version")

    def to_json(self) -> Dict[str, Any]:
        """Translates an ExtensionEnvironmentConfigurationVersion to a JSON dict."""
        return {"version": self.version}

    def put(self, extension_name: str):
        """Updates an existing extension environment config's version in Dynatrace

        :param extension_name: the name of the extension required for making the API call
        """
        return self._http_client.make_request(f"{ExtensionsServiceV2.ENDPOINT}/{extension_name}/environmentConfiguration", params=self.to_json(), method="PUT")


class MinimalExtension(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.version: str = raw_element.get("version")
        self.extension_name: str = raw_element.get("extensionName")

    def list_version(self) -> str:
        """Class method for obtaining extension2.0 version"""
        return self.version

    def get_environment_config(self) -> "ExtensionEnvironmentConfigurationVersion":
        """Gets the active environment configuration version of the specified extension 2.0

        :return: ExtensionEnvironmentConfigurationVersion object
        """
        response = self._http_client.make_request(f"{ExtensionsServiceV2.ENDPOINT}/{self.extension_name}/environmentConfiguration").json()
        return ExtensionEnvironmentConfigurationVersion(raw_element=response)

    def list_environment_config_events(self) -> PaginatedList["ExtensionEventDTO"]:
        """List of the latest extension environment configuration events

        :return: a list of ExtensionEventDTO object
        """
        return PaginatedList(
            ExtensionEventDTO,
            self._http_client,
            target_url=f"{ExtensionsServiceV2.ENDPOINT}/{self.extension_name}/environmentConfiguration/events",
            list_item="extensionEvents",
        )

    def list_monitoring_config_events(self, config_id) -> PaginatedList["ExtensionEventDTO"]:
        """Gets the list of the events linked to specific monitoring configuration

        :param config_id: The ID of the requested monitoring configuration.

        :return: a list of ExtensionEventDTO object
        """
        return PaginatedList(
            ExtensionEventDTO,
            self._http_client,
            target_url=f"{ExtensionsServiceV2.ENDPOINT}/{self.extension_name}/monitoringConfigurations/{config_id}/events",
            list_item="extensionEvents",
        )


class MonitoringConfigurationDto:
    def __init__(self, scope: str, configuration: Dict[str, Any]):
        self.scope = scope
        self.configuration = configuration

    def to_json(self):
        return {"scope": self.scope, "value": self.configuration}



from dynatrace import Dynatrace
from dynatrace.configuration_v1.notifications import NotificationConfigStub, NotificationType, ServiceNowNotificationConfig
from dynatrace.pagination import PaginatedList

ID = "0d06c889-4cea-4b45-aefa-a277790e784d"
NAME = "Service Now Example"
TYPE = NotificationType.SERVICE_NOW


def test_list(dt: Dynatrace):
    notifications = dt.notifications.list()
    assert isinstance(notifications, PaginatedList)

    list_notifications = list(notifications)
    assert len(list_notifications) == 4

    first = list_notifications[0]
    assert isinstance(first, NotificationConfigStub)

    assert first.id == ID
    assert first.name == NAME
    assert first.type == TYPE


def test_get_full_configuration(dt: Dynatrace):
    notifications = dt.notifications.list()
    list_notifications = list(notifications)
    first = list_notifications[0]

    full = first.get_full_configuration()

    # type checks
    assert isinstance(full, ServiceNowNotificationConfig)
    assert isinstance(full.name, str)
    assert isinstance(full.id, str)
    assert isinstance(full.type, NotificationType)
    assert isinstance(full.alerting_profile, str)
    assert isinstance(full.url, type(None))
    assert isinstance(full.username, str)
    assert isinstance(full.password, type(None))
    assert isinstance(full.message, str)
    assert isinstance(full.send_incidents, bool)
    assert isinstance(full.send_events, bool)

    # value checks
    assert full.id == ID
    assert full.name == NAME
    assert full.type == TYPE
    assert full.alerting_profile == "7693d108-fda9-3529-8ca3-55e9269b6097"
    assert full.active == True
    assert full.instance_name == "dev63549"
    assert full.url == None
    assert full.message == "{State} {ProblemID} {ProblemImpact} {ProblemSeverity}"
    assert full.send_incidents == True
    assert full.send_events == False
